@@ -27,11 +27,11 @@ from jax._src import dtypes
 from jax._src.state.types import AbstractRef
 from jax._src.tree_util import (
     PyTreeDef, tree_flatten, tree_unflatten, treedef_children,
-    tree_flatten_with_path, generate_key_paths, broadcast_prefix, prefix_errors,
+    generate_key_paths, broadcast_prefix, prefix_errors,
     none_leaf_registry, broadcast_flattened_prefix_with_treedef)
 from jax._src import linear_util as lu
 from jax._src.util import (safe_map, WrapKwArgs, Hashable, HashableFunction,
-                           Unhashable, safe_zip, unzip2)
+                           Unhashable, safe_zip)
 from jax._src import traceback_util
 
 traceback_util.register_exclusion(__file__)
@@ -76,16 +76,6 @@ def flatten_fun(f: Callable, store: lu.Store,
   store.store(out_tree)
   return ans
 
-@lu.transformation_with_aux2
-def flatten_fun3(f: Callable, store: lu.Store,
-                in_tree: PyTreeDef, *args_flat):
-  py_args, py_kwargs = tree_unflatten(in_tree, args_flat)
-  ans = f(*py_args, **py_kwargs)
-  paths_and_ans, out_tree = tree_flatten_with_path(ans)
-  paths, ans = unzip2(paths_and_ans)
-  store.store((out_tree, paths))
-  return ans
-
 def apply_flat_fun(fun, io_tree, *py_args):
   in_tree_expected, out_tree = io_tree
   args, in_tree = tree_flatten((py_args, {}))
@@ -519,9 +509,6 @@ def resolve_argnums(
   * fills in any missing pieces (e.g., names given numbers, or vice versa),
   * validates the argument names/numbers against the function signature,
   * validates that donated and static arguments don't intersect.
-  * rebases the donated arguments so they index into the dynamic arguments,
-    (after static arguments have been removed), in the order that parameters
-    are passed into the compiled function.
   """
   if signature is None:
     # Some built-in functions don't support signature.
@@ -555,7 +542,6 @@ def resolve_argnums(
 
   # Compensate for static argnums absorbing args
   _assert_no_intersection(static_argnames, donate_argnames)
-  donate_argnums = rebase_donate_argnums(donate_argnums, static_argnums)
   return donate_argnums, donate_argnames, static_argnums, static_argnames
 
 
