@@ -33,7 +33,7 @@ from jax._src.interpreters import batching
 from jax._src.interpreters.batching import not_mapped
 from jax._src.interpreters import mlir
 from jax._src.interpreters import partial_eval as pe
-from jax._src.interpreters import xla
+from jax._src.interpreters import pxla
 from jax._src.tree_util import (tree_flatten, tree_map, tree_structure,
                                 tree_unflatten, treedef_tuple)
 
@@ -260,7 +260,8 @@ def custom_vmap_batching(args_flat, dims, *, call, rule, in_tree, out_tree):
 
 
 def custom_vmap_abstract_eval(*in_avals, call, **_):
-  return call.out_avals
+  del in_avals
+  return call.out_avals, call.effects
 
 
 def custom_vmap_jvp(primals, tangents, *,
@@ -347,10 +348,10 @@ def custom_vmap_jvp(primals, tangents, *,
 custom_vmap_p = core.Primitive('custom_vmap_call')
 custom_vmap_p.multiple_results = True
 custom_vmap_p.def_impl(custom_vmap_impl)
-custom_vmap_p.def_abstract_eval(custom_vmap_abstract_eval)
+custom_vmap_p.def_effectful_abstract_eval(custom_vmap_abstract_eval)
 batching.primitive_batchers[custom_vmap_p] = custom_vmap_batching
 ad.primitive_jvps[custom_vmap_p] = custom_vmap_jvp
-xla.register_initial_style_primitive(custom_vmap_p)
+pxla.register_initial_style_primitive(custom_vmap_p)
 mlir.register_lowering(custom_vmap_p, mlir.lower_fun(
     custom_vmap_impl, multiple_results=True))
 
