@@ -15,6 +15,7 @@
 """Implements SdyShardingRule."""
 
 from collections import OrderedDict
+from typing import Union
 
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import sdy
@@ -28,7 +29,7 @@ BATCHING: str = "…"
 _BATCHING_DIM_FACTOR_PREFIX = "?"
 
 # A Jax value in general corresponds to an ir.Type or a tuple of ir.Types.
-IrTypes = ir.Type | tuple[ir.Type, ...]
+IrTypes = Union[ir.Type, tuple[ir.Type, ...]]
 
 def _check_factor(factor:str):
   """Validates a factor.
@@ -42,6 +43,20 @@ def _check_factor(factor:str):
     if char != "_" and not char.isdigit() and not char.isalpha():
       raise ValueError(f"Unknown character '{char}'")
 
+def _is_batching(factor: str) -> bool:
+  """Checks if a factor is a representation for leading batching dimensions.
+
+  Leading batching dimensions is represented by a factor containing ... and
+     optionally followed by a digit, and ... is equivalent to ...0.
+  """
+  if len(factor) < 1 or factor[0] != BATCHING:
+    return False
+  return len(factor) == 1 or factor[1:].isdigit()
+
+def _get_batching_group(factor: str) -> str:
+  """Extracts the batching group from a factor for leading batching dimensions."""
+  return factor[1:] if len(factor) > 1 else "0"
+
 class CompoundFactor(tuple):
   """Describes the factors for a compound factor.
 
@@ -54,7 +69,7 @@ class CompoundFactor(tuple):
     for factor in factors:
       if not isinstance(factor, str):
         raise ValueError(f"Each element of CompoundFactor must be a str, but got {type(factor)}")
-      if factor == BATCHING:
+      if _is_batching(factor):
         raise ValueError("Ellipsis can't be used in a compound factor")
       else:
         _check_factor(factor)
@@ -80,7 +95,7 @@ class ArrayMapping(tuple):
             "Each element of ArrayMapping must be a str or CompoundFactor, but"
             f" got {type(d)}")
       if isinstance(d, str):
-        if d == BATCHING:
+        if _is_batching(d):
           if i != 0:
             raise ValueError("Ellipsis can only be used at the beginning of a dimension")
         else:
@@ -93,18 +108,30 @@ class ArrayMapping(tuple):
 class SdyShardingRule:
   """Represents a Shardy sharding rule.
 
-  An SdyShardingRule contains the ArrayMappings for operands and results, and an
-  optional list of factor sizes. A factor is a name used in the ArrayMappings.
-  If a factor is only used in CompoundFactors, its size must be specified.
+  An SdyShardingRule contains the ArrayMappings for operands and results,
+  optional special factors and optional factor sizes. A factor is a name used in
+  the ArrayMappings. If a factor is only used in CompoundFactors, its size must
+  be specified.
+
+  By default, a factor is a passthrough factor. Keyword arguments can be used to
+  specify other factor kinds including reduction_factors, need_replication_factors,
+  and permutation_factors.
   """
   operand_mappings: tuple[ArrayMapping, ...]
   result_mappings: tuple[ArrayMapping, ...]
   factor_sizes: dict[str, int]
+  reduction_factors: tuple[str, ...]
+  need_replication_factors: tuple[str, ...]
+  permutation_factors: tuple[str, ...]
 
   def __init__(self, operand_mappings: tuple[ArrayMapping, ...],
-               result_mappings: tuple[ArrayMapping, ...], **factor_sizes):
+               result_mappings: tuple[ArrayMapping, ...],
+               *, reduction_factors: tuple[str, ...] = (),
+               need_replication_factors: tuple[str, ...] = (),
+               permutation_factors: tuple[str, ...] = (),
+               **factor_sizes: int):
     # Find all factors and mark whether their size can be inferred.
-    factors_inferrable = dict()
+    factors_inferrable = {}
     for value in operand_mappings + result_mappings:
       for dim in value:
         if isinstance(dim, str):
@@ -123,25 +150,59 @@ class SdyShardingRule:
     # Check that factors that are used for a whole dimension aren't in
     # factor_sizes and factors that are never used for a whole dimension are
     # in factor_sizes.
-    for factor, inferrable in factors_inferrable.items():
-      if factor not in factor_sizes and not inferrable:
+    for factor, inferable in factors_inferrable.items():
+      if factor not in factor_sizes and not inferable:
         raise ValueError(
           f"Factor {factor} is only used in compound factors; must specify"
           " its size")
-      if factor in factor_sizes and inferrable:
+      if factor in factor_sizes and inferable:
         raise ValueError(
           f"Factor {factor} represents a whole dimension; do not specify its"
           " size")
 
+    special_factors = set()
+    def check_special_factors(kind, factors):
+      if not isinstance(factors, tuple):
+        raise ValueError(f"{kind} must be a tuple of factors")
+
+      if len(factors) != len(set(factors)):
+        raise ValueError(f"{kind} contains duplicated factors")
+
+      for factor in factors:
+        if factor not in factors_inferrable:
+          raise ValueError(
+            f"Factor {factor} in {kind} is not used in the rule")
+        if factor in special_factors:
+          raise ValueError(f"Factor {factor} can only be in one of the "
+              f"reduction, need replication, or permutation factor sets.")
+        special_factors.add(factor)
+
+    check_special_factors("reduction_factors", reduction_factors)
+    check_special_factors("need_replication_factors", need_replication_factors)
+    check_special_factors("permutation_factors", permutation_factors)
+
     self.operand_mappings = operand_mappings
     self.result_mappings = result_mappings
     self.factor_sizes = factor_sizes
+    self.reduction_factors = reduction_factors
+    self.need_replication_factors = need_replication_factors
+    self.permutation_factors = permutation_factors
+
 
   def __str__(self):
-    return f"SdyShardingRule({self.operand_mappings}, {self.result_mappings}, {self.factor_sizes})"
+    def to_str(kind, factors):
+      if len(factors) > 0:
+        return f" {kind}={factors}"
+      return ""
+
+    special_factors = (to_str("reduction_factors", self.reduction_factors) +
+        to_str("need_replication_factors", self.need_replication_factors) +
+        to_str("permutation_factors", self.permutation_factors))
+    return (f"SdyShardingRule({self.operand_mappings}, {self.result_mappings}, "
+            f"{self.factor_sizes}{special_factors})")
 
 
-def _get_batching_dim_factor_name(batch_dim_order : int):
+def _get_batching_dim_factor_name(batch_group: str,batch_dim_order : int):
   """Constructs a factor name for a batching dimension.
 
   We expand the leading ... into factors representing the batching dimensions
@@ -149,7 +210,7 @@ def _get_batching_dim_factor_name(batch_dim_order : int):
   reason, we construct a factor name that won't be used by users for the
   batching dimensions.
   """
-  return f"{_BATCHING_DIM_FACTOR_PREFIX}{batch_dim_order}"
+  return f"{_BATCHING_DIM_FACTOR_PREFIX}{batch_group}_{batch_dim_order}"
 
 def _parse_values(
     rule: str,
@@ -194,13 +255,26 @@ def _parse_values(
     else:
       current_compound_dim.append(x)
 
-  for char in rule:
+  rule_len = len(rule)
+  rule_index = 0
+  while rule_index < rule_len:
+    char = rule[rule_index]
+    rule_index += 1
     if char == BATCHING:
       if (current_factor is not None or current_compound_dim is not None
           or value):
         raise ValueError(
             "Ellipsis can only be used at the beginning of a dimension")
-      add_factor(BATCHING)
+      if rule_index < rule_len and rule[rule_index].isdigit():
+        batching_group_str = ""
+        while rule_index < rule_len and rule[rule_index].isdigit():
+          batching_group_str += rule[rule_index]
+          rule_index += 1
+        batching_group = str(int(batching_group_str))
+      else:
+        batching_group = "0"
+
+      add_factor(f"{BATCHING}{batching_group}")
       continue
     if char in "(), ":
       if current_factor is not None:
@@ -240,15 +314,22 @@ def _parse_values(
 
   return tuple(all_values)
 
-def str_to_sdy_sharding_rule(rule: str, **factor_sizes) -> SdyShardingRule:
+def str_to_sdy_sharding_rule(rule: str, *,
+                             reduction_factors: tuple[str, ...] = (),
+                             need_replication_factors: tuple[str, ...] = (),
+                             permutation_factors: tuple[str, ...] = (),
+                             **factor_sizes: int) -> SdyShardingRule:
   """Constructs a SdyShardingRule object from the Einsum notation like string.
 
   This is done by verifying that the input Einsum notation like string and
-  with optional factor sizes represents a valid sharding rule and converting
-  it to an internal representation.
+  with optional special factors and factor sizes represents a valid sharding
+  rule and converting it to an internal representation.
 
   Args:
     rule: The Einsum notation like string for an operation.
+    reduction_factors: A tuple of factors that are reduction factors.
+    need_replication_factors: A tuple of factors that are need_replication factors.
+    permutation_factors: A tuple of factors that are permutation factors.
     **factor_sizes: The optional factor sizes.
 
   Raises:
@@ -275,8 +356,11 @@ def str_to_sdy_sharding_rule(rule: str, **factor_sizes) -> SdyShardingRule:
 
   operand_mappings = _parse_values(operands)
   result_mappings = _parse_values(results)
-
-  return SdyShardingRule(operand_mappings, result_mappings, **factor_sizes)
+  return SdyShardingRule(operand_mappings, result_mappings,
+                         reduction_factors=reduction_factors,
+                         need_replication_factors=need_replication_factors,
+                         permutation_factors=permutation_factors,
+                         **factor_sizes)
 
 
 def sdy_sharding_rule_to_mlir(
@@ -323,18 +407,20 @@ def sdy_sharding_rule_to_mlir(
 
     `size` may be a dimensions size, a user specified factor size, or UNKNOWN
     if a factor is first used as in a compound factor and then used for a
-    whole dimension.
+    whole dimension. If a factor is not for a leading batching dimension and
+    it corresponds to multiple sizes, the smallest size is used.
     """
     factor_index, factor_size = factors_to_indices_sizes.get(factor, [UNKNOWN, UNKNOWN])
     if factor_index != UNKNOWN:
       # Not the first time seeing the factor.
       if size != UNKNOWN and factor_size != UNKNOWN and factor_size != size:
-        factor_or_batching_dim = (
-          f"Factor {factor}" if _BATCHING_DIM_FACTOR_PREFIX not in factor
-            else f"Batching dimension {factor[1:]}")
-        raise ValueError(
-          f"{factor_or_batching_dim} corresponds to two sizes:"
-          f" {factor_size} and {size}")
+        if _BATCHING_DIM_FACTOR_PREFIX in factor:
+          raise ValueError(f"Batching dimension {factor[1:]} corresponds to "
+                           f"two sizes: {factor_size} and {size}")
+        else:
+          if size < factor_size:
+            # Use the smaller size to update the factor size.
+            factor_size = UNKNOWN
       if size != UNKNOWN and factor_size == UNKNOWN:
         factors_to_indices_sizes[factor] = [factor_index, size]
     else:
@@ -342,9 +428,8 @@ def sdy_sharding_rule_to_mlir(
       factor_index = len(factors_to_indices_sizes)
       factors_to_indices_sizes[factor] = [factor_index, size]
 
-  def add_batching_dim_factor(batch_dim_order, factor_size):
-    ellipsis_batch_dim_name = _get_batching_dim_factor_name(batch_dim_order)
-    add_factor(ellipsis_batch_dim_name, factor_size)
+  def add_batching_dim_factor(batch_grp, batch_dim_order, factor_size):
+    add_factor(_get_batching_dim_factor_name(batch_grp, batch_dim_order), factor_size)
 
   def build_dim_mapping_for_compound_factors(i, j, factors):
     accumulated_size = 1
@@ -363,25 +448,30 @@ def sdy_sharding_rule_to_mlir(
 
     return sdy.DimMappingAttr.get(factor_indices=all_indices)
 
+  def factors_to_indices(factors):
+    return [factors_to_indices_sizes[factor][0] for factor in factors]
+
   # Add factors and their sizes in the order they appear in the rule,
   # including the batching dimensions represented by ellipsis.
-  ellipsis_rank = None
+  batching_group_to_rank: dict[str, int] = {}
   for i, mapping in enumerate(rule.operand_mappings + rule.result_mappings):
     value = tuple(mapping)
-    if value and value[0] == BATCHING:
-      has_batching = True
+    if value and _is_batching(value[0]):
+      batching_group = _get_batching_group(value[0])
       value = value[1:]
     else:
-      has_batching = False
+      batching_group = None
     rule_rank = len(value)
     op_rank = get_rank_for_value(i)
     # The number of dimensions represented by ellipsis.
     current_batching_rank = 0
-    if has_batching and op_rank >= rule_rank:
+    if batching_group is not None and op_rank >= rule_rank:
       current_batching_rank = op_rank - rule_rank
-    if has_batching:
+    if batching_group is not None:
+      ellipsis_rank = batching_group_to_rank.get(batching_group, None)
       if ellipsis_rank is None:
         ellipsis_rank = current_batching_rank
+        batching_group_to_rank[batching_group] = ellipsis_rank
       elif ellipsis_rank != current_batching_rank:
         raise ValueError(
           "Ellipsis represents different number of leading dimensions"
@@ -394,7 +484,7 @@ def sdy_sharding_rule_to_mlir(
         f" {msg} has rank {op_rank}")
 
     for j in range(current_batching_rank):
-      add_batching_dim_factor(j, get_size_for_value_dim(i, j))
+      add_batching_dim_factor(batching_group, j, get_size_for_value_dim(i, j))
 
     for j, dim in enumerate(value):
       if isinstance(dim, str):
@@ -408,20 +498,25 @@ def sdy_sharding_rule_to_mlir(
   for i, mapping in enumerate(rule.operand_mappings + rule.result_mappings):
     value = tuple(mapping)
     dim_mappings = []
-
-    if value and value[0] == BATCHING:
+    if value and _is_batching(value[0]):
+      batching_group = _get_batching_group(value[0])
       value = value[1:]
-      if ellipsis_rank is None:
-        current_batching_rank = 0
+      if batching_group in batching_group_to_rank:
+        #  This type check error is not correct, disable it:
+        # Incompatible types in assignment (expression has type "int | None"
+        current_batching_rank = batching_group_to_rank.get(batching_group) # type: ignore
       else:
-        current_batching_rank = ellipsis_rank
+        raise ValueError("Unreachabled code")
     else:
       current_batching_rank = 0
+      batching_group = None
 
     for j in range(current_batching_rank):
+      #  This type check error is not correct, disable it:
+      # Argument 1 to "_get_batching_dim_factor_name" has incompatible type "str | None"; expected "str"  [arg-type]
       dim_mappings.append(
         sdy.DimMappingAttr.get(factor_indices=[
-          factors_to_indices_sizes[_get_batching_dim_factor_name(j)][0]]))
+          factors_to_indices_sizes[_get_batching_dim_factor_name(batching_group, j)][0]])) # type: ignore
 
     for j, dim in enumerate(value):
       if isinstance(dim, str):
@@ -439,4 +534,9 @@ def sdy_sharding_rule_to_mlir(
   return sdy.OpShardingRuleAttr.get(
       factor_sizes=[item[1] for item in factors_to_indices_sizes.values()],
       operand_mappings=tensor_mappings[0:len(operand_types)],
-      result_mappings=tensor_mappings[len(operand_types):])
+      result_mappings=tensor_mappings[len(operand_types):],
+      is_custom=True,
+      reduction_factors=factors_to_indices(rule.reduction_factors),
+      need_replication_factors=factors_to_indices(rule.need_replication_factors),
+      permutation_factors=factors_to_indices(rule.permutation_factors),
+      )
