@@ -125,14 +125,10 @@ def warn(deprecation_id: str, message: str, stacklevel: int) -> None:
 # always registered by the time `accelerate` and `is_acelerated` are called.
 register('default-dtype-bits-config')
 register('jax-checkpoint-concrete')
-register('jax-lax-dot-positional-args')
-register('jax-lib-module')
 register('jax-nn-one-hot-float-input')
 register('jax-numpy-arange-complex')
 register('jax-numpy-astype-complex-to-real')
 register('jax-numpy-clip-args')
 register('jax-scipy-special-sph-harm')
-register('safer-randint-config')
-register('jax-pmap-no-rank-reduction')
-register('jax-make-mesh-default-explicit')
+register('jax-pmap-shmap-merge')
 register('pltpu-memory-space-any')
