@@ -12,13 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable
-from collections.abc import Hashable
-
-from jax import Array
+from collections.abc import Callable, Hashable
 
 from jax._src import prng
 from jax._src import random
+from jax._src.typing import Array
 
 Shape = tuple[int, ...]
 
