@@ -47,3 +47,6 @@ class FrozenDict(Mapping[K, V]):
 
   def __len__(self) -> int:
     return len(self._d)
+
+  def get(self, key: K) -> V | None:  # type: ignore
+    return self._d.get(key, None)
