@@ -78,12 +78,11 @@ from jax._src import ad_util
 from jax._src import core
 from jax._src import dispatch
 from jax._src import dtypes
+from jax._src.numpy.indexing import take_along_axis
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
 from jax._src.interpreters import mlir
-from jax._src.interpreters import xla
-from jax._src.lax import lax
-from jax._src.lib import xla_client as xc
+from jax._src.lib import _jax
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import func
 from jax._src.lib.mlir.dialects import hlo
@@ -232,7 +231,7 @@ def _approx_top_k_abstract_eval(operand, *, k, reduction_dimension,
   if aggregate_to_topk:
     dims[reduction_dimension] = k
   elif core.is_constant_shape((reduction_input_size, k)):
-    dims[reduction_dimension] = xc.ops.ApproxTopKReductionOutputSize(
+    dims[reduction_dimension] = _jax.approx_top_k_reduction_output_size(
         reduction_input_size, len(dims), k, recall_target, aggregate_to_topk,
         reduction_input_size_override)[0]
   else:
@@ -240,24 +239,17 @@ def _approx_top_k_abstract_eval(operand, *, k, reduction_dimension,
          "approx_top_k with aggregate_to_topk=False not yet implemented when "
          f"either the `k` ({k}) or the "
          f" reduction dimension size ({reduction_input_size}) are symbolic")
+  operand_s = operand.sharding
+  if operand_s.spec[reduction_dimension] is not None:
+    raise core.ShardingTypeError(
+        f"reduction dimension {reduction_dimension} in operand"
+        f" {operand.str_short()} should be unsharded i.e. the spec of that dim"
+        " should be `None`.")
   return (operand.update(shape=dims, dtype=operand.dtype,
-                         weak_type=operand.weak_type),
-          operand.update(shape=dims, dtype=np.dtype(np.int32)))
-
-
-def _comparator_builder(op_type, is_max_k):
-  c = xc.XlaBuilder(
-      'top_k_{}_comparator'.format('gt' if is_max_k else 'lt'))
-  p0 = xla.parameter(c, 0, xc.Shape.scalar_shape(op_type))
-  p1 = xla.parameter(c, 1, xc.Shape.scalar_shape(op_type))
-  xla.parameter(c, 2, xc.Shape.scalar_shape(np.dtype(np.int32)))
-  xla.parameter(c, 3, xc.Shape.scalar_shape(np.dtype(np.int32)))
-  if is_max_k:
-    cmp_result = xc.ops.Gt(p0, p1)
-  else:
-    cmp_result = xc.ops.Lt(p0, p1)
-  return c.build(cmp_result)
-
+                         weak_type=operand.weak_type, vma=operand.vma,
+                         sharding=operand_s),
+          operand.update(shape=dims, dtype=np.dtype(np.int32), vma=operand.vma,
+                         sharding=operand_s))
 
 def _get_init_val_literal(op_type, is_max_k):
   return np.array(-np.inf if is_max_k else np.inf, dtype=op_type)
@@ -335,11 +327,6 @@ def _approx_top_k_lowering(ctx, operand, *, k,
         backend_config=backend_config,
         result_shapes=result_shapes)
   else:
-    if xc.mlir_api_version < 57:
-      raise NotImplementedError(
-          "approx_top_k with non-constant k requires jaxlib version 0.4.29 or "
-          "newer")
-
     k_value, = mlir.eval_dynamic_shape_as_vals(ctx, (k,))
     out = mlir.custom_call(
         "stablehlo.dynamic_approx_top_k",
@@ -394,19 +381,14 @@ def _approx_top_k_jvp(primals, tangents, *, k, reduction_dimension,
                                     reduction_input_size_override,
                                     aggregate_to_topk)
   if type(tangent) is ad_util.Zero:
-    tangent_out = ad_util.Zero.from_value(val_out)
+    tangent_out = ad_util.Zero.from_primal_value(val_out)
   else:
     arg_shape = arg_out.shape
     rank = len(arg_shape)
     if reduction_dimension < 0:
       reduction_dimension += rank
-    iotas = [
-        lax.broadcasted_iota(arg_out.dtype, arg_shape, i) for i in range(rank)
-    ]
-    idx = tuple(
-        arg_out if i == reduction_dimension else iotas[i] for i in range(rank))
-    tangent_out = tangent[idx]
-  return (val_out, arg_out), (tangent_out, ad_util.Zero.from_value(arg_out))
+    tangent_out = take_along_axis(tangent, arg_out, axis=reduction_dimension)
+  return (val_out, arg_out), (tangent_out, ad_util.Zero.from_primal_value(arg_out))
 
 
 approx_top_k_p = core.Primitive('approx_top_k')
