@@ -11,13 +11,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Definitions of Mesh and ResourceEnv."""
+"""Definitions of Mesh and AbstractMesh"""
 
 from __future__ import annotations
 
 import collections
 from collections.abc import Hashable, Sequence
 import contextlib
+import dataclasses
 import enum
 import functools
 import math
@@ -32,6 +33,7 @@ from jax._src.util import safe_zip, cache, tuple_delete
 from jax._src.lib import xla_client as xc
 
 zip, unsafe_zip = safe_zip, zip
+config_ext = xc._xla.config
 
 MeshAxisName = Any
 ResourceAxisName = Hashable
@@ -141,77 +143,71 @@ def any_axis_types_match(axis_types, ty: AxisType) -> bool:
 class BaseMesh:
   axis_names: tuple[MeshAxisName, ...]
   shape_tuple: tuple[tuple[str, int], ...]
-  _axis_types: tuple[AxisType, ...]
-
-  @property
-  def axis_types(self) -> tuple[AxisType, ...]:
-    return self._axis_types
+  axis_types: tuple[AxisType, ...]
 
   @functools.cached_property
-  def _are_all_axes_manual(self) -> bool:
-    return all_axis_types_match(self._axis_types, AxisType.Manual)
+  def are_all_axes_manual(self) -> bool:
+    return all_axis_types_match(self.axis_types, AxisType.Manual)
 
   @functools.cached_property
-  def _are_all_axes_auto(self) -> bool:
-    return all_axis_types_match(self._axis_types, AxisType.Auto)
+  def are_all_axes_auto(self) -> bool:
+    return all_axis_types_match(self.axis_types, AxisType.Auto)
 
   @functools.cached_property
-  def _are_all_axes_explicit(self) -> bool:
-    return all_axis_types_match(self._axis_types, AxisType.Explicit)
+  def are_all_axes_explicit(self) -> bool:
+    return all_axis_types_match(self.axis_types, AxisType.Explicit)
 
   @functools.cached_property
   def _are_all_axes_auto_or_manual(self) -> bool:
-    if not self._axis_types:
+    if not self.axis_types:
       return False
     return all(t == AxisType.Auto or t == AxisType.Manual
-               for t in self._axis_types)
+               for t in self.axis_types)
+
+  @functools.cached_property
+  def _are_all_axes_explicit_or_manual(self) -> bool:
+    if not self.axis_types:
+      return False
+    return all(t == AxisType.Explicit or t == AxisType.Manual
+               for t in self.axis_types)
 
   @functools.cached_property
   def _any_axis_manual(self) -> bool:
-    return any_axis_types_match(self._axis_types, AxisType.Manual)
+    return any_axis_types_match(self.axis_types, AxisType.Manual)
 
   @functools.cached_property
   def _any_axis_auto(self) -> bool:
-    return any_axis_types_match(self._axis_types, AxisType.Auto)
+    return any_axis_types_match(self.axis_types, AxisType.Auto)
 
   @functools.cached_property
   def _any_axis_explicit(self) -> bool:
-    return any_axis_types_match(self._axis_types, AxisType.Explicit)
+    return any_axis_types_match(self.axis_types, AxisType.Explicit)
 
   @functools.cached_property
   def _any_axis_auto_or_manual(self) -> bool:
-    if not self._axis_types:
+    if not self.axis_types:
       return False
     return any(t == AxisType.Auto or t == AxisType.Manual
-               for t in self._axis_types)
+               for t in self.axis_types)
 
   @functools.cached_property
   def auto_axes(self):
-    return tuple(n for n, t in safe_zip(self.axis_names, self._axis_types)
+    return tuple(n for n, t in safe_zip(self.axis_names, self.axis_types)
                  if t == AxisType.Auto)
 
   @functools.cached_property
   def explicit_axes(self):
-    return tuple(n for n, t in safe_zip(self.axis_names, self._axis_types)
+    return tuple(n for n, t in safe_zip(self.axis_names, self.axis_types)
                  if t == AxisType.Explicit)
 
   @functools.cached_property
   def manual_axes(self):
-    return tuple(n for n, t in safe_zip(self.axis_names, self._axis_types)
+    return tuple(n for n, t in safe_zip(self.axis_names, self.axis_types)
                  if t == AxisType.Manual)
 
-  @functools.cached_property
-  def _axis_types_dict(self):
-    if not self.axis_names:
-      return {}
-    d = collections.defaultdict(list)
-    for n, t in safe_zip(self.axis_names, self._axis_types):
-      d[t].append(n)
-    return {t: tuple(n) for t, n in d.items()}
-
   @functools.cached_property
   def _name_to_type(self):
-    return dict(safe_zip(self.axis_names, self._axis_types))
+    return dict(safe_zip(self.axis_names, self.axis_types))
 
 
 def _unpicke_mesh(devices, axis_names, axis_types):
@@ -285,13 +281,13 @@ class Mesh(BaseMesh, contextlib.ContextDecorator):
     self.devices = devices.copy()
     self.devices.flags.writeable = False
     self.axis_names = axis_names
-    self._axis_types = axis_types
+    self.axis_types = axis_types
     self._size = math.prod(self.shape.values()) if self.devices.ndim else 0
     _mesh_object_dict[key] = self
     return self
 
   def __reduce__(self):
-    return (_unpicke_mesh, (self.devices, self.axis_names, self._axis_types))
+    return (_unpicke_mesh, (self.devices, self.axis_names, self.axis_types))
 
   def __eq__(self, other):
     # This is a performance optimization. Comparing thousands of devices
@@ -302,14 +298,14 @@ class Mesh(BaseMesh, contextlib.ContextDecorator):
       return False
     return (self.axis_names == other.axis_names and
             self.devices.shape == other.devices.shape and
-            self._axis_types == other._axis_types and
+            self.axis_types == other.axis_types and
             self._internal_device_list == other._internal_device_list)
 
   def __hash__(self):
     if not hasattr(self, '_hash'):
       self._hash = hash(
           (self.axis_names, self._internal_device_list, self.devices.shape,
-           self._axis_types))
+           self.axis_types))
     return self._hash
 
   def __setattr__(self, name, value):
@@ -342,6 +338,15 @@ class Mesh(BaseMesh, contextlib.ContextDecorator):
               if not t.physical_mesh.empty))
     return False
 
+  def update(self, devices=None, axis_names=None, axis_types=None):
+    if devices is None:
+      devices = self.devices
+    if axis_names is None:
+      axis_names = self.axis_names
+    if axis_types is None:
+      axis_types = self.axis_types
+    return Mesh(devices, axis_names, axis_types)
+
   @functools.cached_property
   def shape(self):
     return collections.OrderedDict(
@@ -399,15 +404,17 @@ class Mesh(BaseMesh, contextlib.ContextDecorator):
     return set(self.devices.flat)
 
   def __str__(self):
+    if self.empty:
+      return "Mesh()"
     mesh_str = ", ".join(f"'{k}': {v}" for k, v in self.shape.items())
-    atr = f", axis_types={self._axis_types}"
+    atr = f", axis_types={self.axis_types}"
     return f"Mesh({mesh_str}{atr})"
 
   @functools.cached_property
   def _repr(self):
     if self.empty:
       return "Mesh(axis_sizes=(), axis_names=())"
-    atr = f", axis_types={self._axis_types}"
+    atr = f", axis_types={self.axis_types}"
     return (f"Mesh(axis_sizes={self.device_ids.shape}, "
             f"axis_names={self.axis_names!r}{atr})")
 
@@ -421,8 +428,21 @@ class Mesh(BaseMesh, contextlib.ContextDecorator):
 
   @functools.cached_property
   def abstract_mesh(self):
-    return AbstractMesh(self.axis_sizes, self.axis_names,
-                        axis_types=self._axis_types)
+    d = self.devices.flat[0]
+    if d is None:
+      abstract_device = None
+    else:
+      if d.platform == 'tpu':
+        num_cores = getattr(d, 'num_cores', None)
+      elif d.platform == 'gpu':
+        num_cores = getattr(d, 'core_count', None)
+      else:
+        num_cores = None
+      abstract_device = AbstractDevice(
+          device_kind=d.device_kind, num_cores=num_cores)
+    return AbstractMesh(
+        self.axis_sizes, self.axis_names, axis_types=self.axis_types,
+        abstract_device=abstract_device)
 
 
 EMPTY_ENV = ResourceEnv(Mesh(np.empty((), dtype=object), ()))
@@ -436,6 +456,18 @@ class _ThreadResourcesLocalState(threading.local):
 thread_resources = _ThreadResourcesLocalState()
 
 
+@dataclasses.dataclass(frozen=True)
+class AbstractDevice:
+  device_kind: str
+  num_cores: int | None
+
+  def __repr__(self):
+    return (f"AbstractDevice({self._repr()})")
+
+  def _repr(self):
+    return f"device_kind={self.device_kind}, num_cores={self.num_cores}"
+
+
 class AbstractMesh(BaseMesh):
   """AbstractMesh contains only axis names and axis sizes.
 
@@ -458,13 +490,16 @@ class AbstractMesh(BaseMesh):
   """
 
   def __init__(self, axis_sizes: tuple[int, ...], axis_names: tuple[str, ...],
-               axis_types: AxisType | tuple[AxisType, ...] | None = None):
+               axis_types: AxisType | tuple[AxisType, ...] | None = None,
+               *, abstract_device=None):
     self.axis_sizes = axis_sizes
     self.axis_names = axis_names
-    self._size = math.prod(self.axis_sizes) if self.axis_sizes else 0
-    self._axis_types = _normalize_axis_types(
+    self.axis_types = _normalize_axis_types(
         self.axis_names, axis_types, 'AbstractMesh')
-    self._hash = hash((self.axis_sizes, self.axis_names, self._axis_types))
+    self.abstract_device = abstract_device
+    self.size = math.prod(self.axis_sizes) if self.axis_sizes else 0
+    self._hash = hash((self.axis_sizes, self.axis_names, self.axis_types,
+                       self.abstract_device))
 
   def __hash__(self):
     return self._hash
@@ -476,17 +511,27 @@ class AbstractMesh(BaseMesh):
       return False
     return (self.axis_sizes == other.axis_sizes and
             self.axis_names == other.axis_names and
-            self._axis_types == other._axis_types)
+            self.axis_types == other.axis_types and
+            self.abstract_device == other.abstract_device)
 
   def __repr__(self):
     mesh_repr = (", ".join(f"'{n}': {v}" for n, v in self.shape_tuple)
                  if self.shape_tuple else "()")
-    atr = f", axis_types={self._axis_types}"
-    return f"AbstractMesh({mesh_repr}{atr})"
+    atr = f", axis_types={self.axis_types}"
+    ad = ("" if self.abstract_device is None else
+          f", {self.abstract_device._repr()}")
+    return f"AbstractMesh({mesh_repr}{atr}{ad})"
 
-  @property
-  def size(self):
-    return self._size
+  def update(self, axis_sizes=None, axis_names=None, axis_types=None, **kwargs):
+    if axis_sizes is None:
+      axis_sizes = self.axis_sizes
+    if axis_names is None:
+      axis_names = self.axis_names
+    if axis_types is None:
+      axis_types = self.axis_types
+    if 'abstract_device' not in kwargs:
+      kwargs['abstract_device'] = self.abstract_device
+    return AbstractMesh(axis_sizes, axis_names, axis_types, **kwargs)
 
   @functools.cached_property
   def shape(self):
@@ -512,9 +557,8 @@ class AbstractMesh(BaseMesh):
 
   def update_axis_types(self, name_to_type: dict[MeshAxisName, AxisType]):
     new_axis_types = tuple(name_to_type[n] if n in name_to_type else a
-                           for n, a in zip(self.axis_names, self._axis_types))
-    return AbstractMesh(self.axis_sizes, self.axis_names,
-                        axis_types=new_axis_types)
+                           for n, a in zip(self.axis_names, self.axis_types))
+    return self.update(axis_types=new_axis_types)
 
   @property
   def devices(self):
@@ -542,11 +586,6 @@ class AbstractMesh(BaseMesh):
   def __exit__(self, exc_type, exc_value, traceback):
     _raise_value_error("__exit__")
 
-  @staticmethod
-  def _extremely_unsafe_enter_tracing_context(mesh: AbstractMesh):
-    prev = jax_config.abstract_mesh_context_manager.swap_local(mesh)
-    return prev
-
 
 # Create this indirection because pytype fails to recognize a property if a
 # property raises an exception unconditionally. Remove this once that is fixed.
@@ -554,8 +593,9 @@ def _raise_value_error(name):
   raise ValueError(f"AbstractMesh does not implement {name}")
 
 empty_abstract_mesh = AbstractMesh((), ())
+empty_concrete_mesh = Mesh(np.empty((), dtype=object), ())
 
-class UseAbstractMeshContextManager:
+class use_abstract_mesh:
   __slots__ = ['mesh', 'prev']
 
   def __init__(self, mesh: AbstractMesh):
@@ -567,15 +607,23 @@ class UseAbstractMeshContextManager:
 
   def __enter__(self):
     self.prev = jax_config.abstract_mesh_context_manager.swap_local(self.mesh)
+    if (self.prev is not config_ext.unset and
+        not self.prev.empty and not self.mesh.empty and
+        self.prev.size != self.mesh.size):
+      jax_config.abstract_mesh_context_manager.set_local(self.prev)
+      raise ValueError(
+          "use_abstract_mesh cannot change the size of the mesh. Got new mesh:"
+          f" {self.mesh} with size={self.mesh.size} and prev mesh:"
+          f" {self.prev} with size={self.prev.size}")
 
   def __exit__(self, exc_type, exc_value, traceback):
     jax_config.abstract_mesh_context_manager.set_local(self.prev)
 
-use_abstract_mesh = UseAbstractMeshContextManager
 
 def get_abstract_mesh() -> AbstractMesh:
   val = jax_config.abstract_mesh_context_manager.value
   return empty_abstract_mesh if val is None else val
 
-def get_concrete_mesh() -> Mesh | None:
-  return jax_config.device_context.value
+def get_concrete_mesh() -> Mesh:
+  val = jax_config.device_context.value
+  return empty_concrete_mesh if val is None else val
