@@ -28,10 +28,12 @@ import numpy as np
 _TPU_V2 = 'TPU v2'
 _TPU_V3 = 'TPU v3'
 _TPU_V4 = 'TPU v4'
+_TPU_V4_LITE = "TPU v4 lite"
 _TPU_V5_LITE = "TPU v5 lite"
 _TPU_V5E = "TPU v5e"
 _TPU_V5P = "TPU v5p"
 _TPU_V6_LITE = "TPU v6 lite"
+_TPU_7X = "TPU7x"
 
 # Maps physical topology -> mesh shape -> transpose to use for jekbradbury's
 # famous contiguous mesh trick.
@@ -70,6 +72,8 @@ _TRAY_4x4_RING_ORDER = (0, 1, 2, 3, 7, 6, 5, 9, 10, 11, 15, 14, 13, 12, 8, 4)
 _V5E_TRAY_RING_ORDER = (0, 1, 2, 3, 7, 6, 5, 4)
 _V5E_TRAY_IOTA_ORDER = (0, 4, 2, 6, 1, 5, 3, 7)
 _V5P_2x2x2_ORDER = (0, 1, 3, 2, 6, 7, 5, 4)
+_7X_TRAY_2x2x2_RING_ORDER = (0, 1, 2, 3, 6, 7, 4, 5)
+
 
 def _tpu_v2_v3_create_device_mesh(
     mesh_shape: Sequence[int],
@@ -93,6 +97,19 @@ def _tpu_v2_v3_create_device_mesh(
     return np.asarray(devices).reshape(mesh_shape)
 
 
+# TODO(b/303712469): Unit test these handler functions.
+# Creates a physical ring 0->1->3->2 if on v4i.
+def _v4i_create_device_mesh(
+    mesh_shape: Sequence[int], devices: Sequence[Any], **unused_kwargs
+) -> np.ndarray | None:
+  if len(devices) == 4:
+    device_mesh = np.asarray(devices)
+    device_mesh = device_mesh[np.array(_TRAY_2x2_RING_ORDER)]
+    device_mesh = device_mesh.reshape(mesh_shape)
+    return device_mesh
+  return None
+
+
 def _v5e_create_device_mesh(
     mesh_shape: Sequence[int], devices: Sequence[Any], **unused_kwargs
 ) -> np.ndarray | None:
@@ -171,6 +188,43 @@ def _v5p_create_device_mesh(
     return device_mesh
   return None
 
+def _7x_create_device_mesh(
+    mesh_shape: Sequence[int], devices: Sequence[Any], **unused_kwargs
+) -> np.ndarray | None:
+  """Creates device assignment for small 7x topologies.
+
+  The device assignment attempts to minimize the number of hops between
+  neighbors by allocating rings of devices, and assigns the core axis
+  preferentially due to its higher bandwidth.
+
+  Args:
+    mesh_shape: Logical mesh shape used by the model.
+    devices: TPU devices.
+    **unused_kwargs: ...
+
+  Returns:
+    None or reordered devices reshaped as `mesh_shape`.
+  """
+  if len(devices) % 8 != 0 or len(devices) > 32:
+    return None
+
+  physical_mesh_shape = _get_physical_tpu_mesh(devices).shape
+  # For the x and y axes, we only support at most 2x2 since we can make one ring
+  # along those axes and repeat with other separate rings along the z axis.
+  if physical_mesh_shape[0] > 2 or physical_mesh_shape[1] > 2:
+    return None
+
+  indices = []
+  for i in range(0, len(devices), 8):
+    new_indices = [x + i for x in _7X_TRAY_2x2x2_RING_ORDER]
+    indices.extend(new_indices)
+
+  device_mesh = np.asarray(devices)
+  device_mesh = device_mesh[np.array(indices)]
+  device_mesh = device_mesh.reshape(mesh_shape)
+  return device_mesh
+
+
 # Registers functions to create device mesh for specific device kinds. Takes
 # precedence over the more general logic in create_device_mesh(). Handler may
 # return None; in that case, it will fall back to using the default logic.
@@ -180,9 +234,11 @@ device_kind_handler_dict: dict[
 ] = {
     _TPU_V2: _tpu_v2_v3_create_device_mesh,
     _TPU_V3: _tpu_v2_v3_create_device_mesh,
+    _TPU_V4_LITE: _v4i_create_device_mesh,
     _TPU_V5_LITE: _v5e_create_device_mesh,
     _TPU_V5P: _v5p_create_device_mesh,
     _TPU_V6_LITE: _v5e_create_device_mesh,
+    _TPU_7X: _7x_create_device_mesh,
 }
 
 
@@ -246,7 +302,7 @@ def _create_device_mesh_for_nd_torus(
       list(enumerate(mesh_shape))
   ):
     # Preferentially map to more physical axes first for higher bandwidth.
-    for num_axes in range(3, 0, -1):
+    for num_axes in range(len(physical_mesh.shape), 0, -1):
       # Try assign to any subset of size num_axes. Generate all candidates.
       indices_and_axes = itertools.combinations(
           enumerate(assignable_physical_mesh), num_axes
@@ -645,6 +701,17 @@ def _get_physical_tpu_mesh(jax_devices: Sequence[Any]) -> np.ndarray:
           coords[1] - min_coords[1],
           d.core_on_chip - min_cores_per_chip,
       ] = d
+  elif (device_kind in (_TPU_7X,) or
+        (device_kind in (_TPU_V5P,) and cores_per_chip == 2)):
+    out = np.empty(dims + (cores_per_chip,), dtype=object)
+    for d in jax_devices:
+      coords = d.coords
+      out[
+          coords[0] - min_coords[0],
+          coords[1] - min_coords[1],
+          coords[2] - min_coords[2],
+          d.core_on_chip - min_cores_per_chip,
+      ] = d
   else:
     out = np.empty(dims, dtype=object)
     for d in jax_devices:
