@@ -13,7 +13,6 @@
 # limitations under the License.
 
 import types
-from functools import partial
 import operator
 from typing import Any, Literal, overload
 
@@ -87,11 +86,12 @@ def zeros(shape: Any, dtype: DTypeLike | None = None, *,
   if isinstance(shape, types.GeneratorType):
     raise TypeError("expected sequence object with len >= 0 or a single integer")
   if (m := _check_forgot_shape_tuple("zeros", shape, dtype)): raise TypeError(m)
-  dtypes.check_user_dtype_supported(dtype, "zeros")
+  dtype = dtypes.check_and_canonicalize_user_dtype(
+      float if dtype is None else dtype, "zeros")
   shape = canonicalize_shape(shape)
   sharding = util.choose_device_or_out_sharding(
       device, out_sharding, 'jnp.zeros')
-  return lax.full(shape, 0, dtypes.jax_dtype(dtype), sharding=sharding)
+  return lax.full(shape, 0, dtype, sharding=sharding)
 
 
 @export
@@ -136,10 +136,11 @@ def ones(shape: Any, dtype: DTypeLike | None = None, *,
     raise TypeError("expected sequence object with len >= 0 or a single integer")
   if (m := _check_forgot_shape_tuple("ones", shape, dtype)): raise TypeError(m)
   shape = canonicalize_shape(shape)
-  dtypes.check_user_dtype_supported(dtype, "ones")
+  dtype = dtypes.check_and_canonicalize_user_dtype(
+      float if dtype is None else dtype, "ones")
   sharding = util.choose_device_or_out_sharding(
       device, out_sharding, 'jnp.ones')
-  return lax.full(shape, 1, dtypes.jax_dtype(dtype), sharding=sharding)
+  return lax.full(shape, 1, dtype, sharding=sharding)
 
 
 @export
@@ -183,7 +184,8 @@ def empty(shape: Any, dtype: DTypeLike | None = None, *,
   .. _explicit sharding: https://docs.jax.dev/en/latest/notebooks/explicit-sharding.html
   """
   if (m := _check_forgot_shape_tuple("empty", shape, dtype)): raise TypeError(m)
-  dtypes.check_user_dtype_supported(dtype, "empty")
+  dtype = dtypes.check_and_canonicalize_user_dtype(
+      float if dtype is None else dtype, "empty")
   return zeros(shape, dtype, device=device, out_sharding=out_sharding)
 
 
@@ -233,13 +235,14 @@ def full(shape: Any, fill_value: ArrayLike,
     Array([[0, 1, 2],
            [0, 1, 2]], dtype=int32)
   """
-  dtypes.check_user_dtype_supported(dtype, "full")
+  if dtype is not None:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "full")
   util.check_arraylike("full", fill_value)
 
   if np.ndim(fill_value) == 0:
     shape = canonicalize_shape(shape)
     return lax.full(shape, fill_value, dtype,
-                    sharding=util.normalize_device_to_sharding(device))
+                    sharding=util.canonicalize_device_to_sharding(device))
   else:
     return api.device_put(
         util._broadcast_to(asarray(fill_value, dtype=dtype), shape), device)
@@ -249,7 +252,8 @@ def full(shape: Any, fill_value: ArrayLike,
 def zeros_like(a: ArrayLike | DuckTypedArray,
                dtype: DTypeLike | None = None,
                shape: Any = None, *,
-               device: xc.Device | Sharding | None = None) -> Array:
+               device: xc.Device | Sharding | None = None,
+               out_sharding: NamedSharding | P | None = None) -> Array:
   """Create an array full of zeros with the same shape and dtype as an array.
 
   JAX implementation of :func:`numpy.zeros_like`.
@@ -284,18 +288,21 @@ def zeros_like(a: ArrayLike | DuckTypedArray,
     if hasattr(a, '__jax_array__'):
       a = a.__jax_array__()
     util.check_arraylike("zeros_like", a)
-  dtypes.check_user_dtype_supported(dtype, "zeros_like")
+  if dtype is not None:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "zeros_like")
   if shape is not None:
     shape = canonicalize_shape(shape)
-  return lax.full_like(a, 0, dtype, shape,
-                       sharding=util.normalize_device_to_sharding(device))
+  sharding = util.choose_device_or_out_sharding(
+      device, out_sharding, "jnp.zeros_like")
+  return lax.full_like(a, 0, dtype, shape, sharding=sharding)
 
 
 @export
 def ones_like(a: ArrayLike | DuckTypedArray,
               dtype: DTypeLike | None = None,
               shape: Any = None, *,
-              device: xc.Device | Sharding | None = None) -> Array:
+              device: xc.Device | Sharding | None = None,
+              out_sharding: NamedSharding | P | None = None) -> Array:
   """Create an array of ones with the same shape and dtype as an array.
 
   JAX implementation of :func:`numpy.ones_like`.
@@ -330,11 +337,13 @@ def ones_like(a: ArrayLike | DuckTypedArray,
     if hasattr(a, '__jax_array__'):
       a = a.__jax_array__()
     util.check_arraylike("ones_like", a)
-  dtypes.check_user_dtype_supported(dtype, "ones_like")
+  if dtype is not None:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "ones_like")
   if shape is not None:
     shape = canonicalize_shape(shape)
-  return lax.full_like(a, 1, dtype, shape,
-                       sharding=util.normalize_device_to_sharding(device))
+  sharding = util.choose_device_or_out_sharding(
+      device, out_sharding, "jnp.ones_like")
+  return lax.full_like(a, 1, dtype, shape, sharding=sharding)
 
 
 @export
@@ -378,11 +387,12 @@ def empty_like(prototype: ArrayLike | DuckTypedArray,
     if hasattr(prototype, '__jax_array__'):
       prototype = prototype.__jax_array__()
     util.check_arraylike("ones_like", prototype)
-  dtypes.check_user_dtype_supported(dtype, "ones_like")
+  if dtype is not None:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "ones_like")
   if shape is not None:
     shape = canonicalize_shape(shape)
   return lax.full_like(prototype, 0, dtype, shape,
-                       sharding=util.normalize_device_to_sharding(device))
+                       sharding=util.canonicalize_device_to_sharding(device))
 
 
 @export
@@ -432,12 +442,13 @@ def full_like(a: ArrayLike | DuckTypedArray,
     util.check_arraylike("full_like", a, fill_value)
     if hasattr(a, '__jax_array__'):
       a = a.__jax_array__()
-  dtypes.check_user_dtype_supported(dtype, "full_like")
+  if dtype is not None:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "full_like")
   if shape is not None:
     shape = canonicalize_shape(shape)
   if np.ndim(fill_value) == 0:
     return lax.full_like(a, fill_value, dtype, shape,
-                         sharding=util.normalize_device_to_sharding(device))
+                         sharding=util.canonicalize_device_to_sharding(device))
   else:
     shape = np.shape(a) if shape is None else shape  # type: ignore[arg-type]
     dtype = dtypes.result_type(a) if dtype is None else dtype
@@ -536,21 +547,21 @@ def linspace(start: ArrayLike, stop: ArrayLike, num: int = 50,
   axis = core.concrete_or_error(operator.index, axis, "'axis' argument of jnp.linspace")
   return _linspace(start, stop, num, endpoint, retstep, dtype, axis, device=device)
 
-@partial(api.jit, static_argnames=('num', 'endpoint', 'retstep', 'dtype', 'axis', 'device'))
+@api.jit(static_argnames=('num', 'endpoint', 'retstep', 'dtype', 'axis', 'device'))
 def _linspace(start: ArrayLike, stop: ArrayLike, num: int = 50,
               endpoint: bool = True, retstep: bool = False,
               dtype: DTypeLike | None = None,
               axis: int = 0,
               *, device: xc.Device | Sharding | None = None) -> Array | tuple[Array, Array]:
   """Implementation of linspace differentiable in start and stop args."""
-  dtypes.check_user_dtype_supported(dtype, "linspace")
   if num < 0:
     raise ValueError(f"Number of samples, {num}, must be non-negative.")
   start, stop = util.ensure_arraylike("linspace", start, stop)
 
   if dtype is None:
     dtype = dtypes.to_inexact_dtype(dtypes.result_type(start, stop))
-  dtype = dtypes.jax_dtype(dtype)
+  else:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "linspace")
   computation_dtype = dtypes.to_inexact_dtype(dtype)
   start = start.astype(computation_dtype)
   stop = stop.astype(computation_dtype)
@@ -662,15 +673,15 @@ def logspace(start: ArrayLike, stop: ArrayLike, num: int = 50,
   axis = core.concrete_or_error(operator.index, axis, "'axis' argument of jnp.logspace")
   return _logspace(start, stop, num, endpoint, base, dtype, axis)
 
-@partial(api.jit, static_argnames=('num', 'endpoint', 'dtype', 'axis'))
+@api.jit(static_argnames=('num', 'endpoint', 'dtype', 'axis'))
 def _logspace(start: ArrayLike, stop: ArrayLike, num: int = 50,
               endpoint: bool = True, base: ArrayLike = 10.0,
               dtype: DTypeLike | None = None, axis: int = 0) -> Array:
   """Implementation of logspace differentiable in start and stop args."""
-  dtypes.check_user_dtype_supported(dtype, "logspace")
   if dtype is None:
     dtype = dtypes.to_inexact_dtype(dtypes.result_type(start, stop))
-  dtype = dtypes.jax_dtype(dtype)
+  else:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "logspace")
   computation_dtype = dtypes.to_inexact_dtype(dtype)
   start, stop = util.ensure_arraylike("logspace", start, stop)
   start = start.astype(computation_dtype)
@@ -733,14 +744,14 @@ def geomspace(start: ArrayLike, stop: ArrayLike, num: int = 50, endpoint: bool =
   axis = core.concrete_or_error(operator.index, axis, "'axis' argument of jnp.geomspace")
   return _geomspace(start, stop, num, endpoint, dtype, axis)
 
-@partial(api.jit, static_argnames=('num', 'endpoint', 'dtype', 'axis'))
+@api.jit(static_argnames=('num', 'endpoint', 'dtype', 'axis'))
 def _geomspace(start: ArrayLike, stop: ArrayLike, num: int = 50, endpoint: bool = True,
                dtype: DTypeLike | None = None, axis: int = 0) -> Array:
   """Implementation of geomspace differentiable in start and stop args."""
-  dtypes.check_user_dtype_supported(dtype, "geomspace")
   if dtype is None:
     dtype = dtypes.to_inexact_dtype(dtypes.result_type(start, stop))
-  dtype = dtypes.jax_dtype(dtype)
+  else:
+    dtype = dtypes.check_and_canonicalize_user_dtype(dtype, "geomspace")
   computation_dtype = dtypes.to_inexact_dtype(dtype)
   start, stop = util.ensure_arraylike("geomspace", start, stop)
   start = start.astype(computation_dtype)
