@@ -16,9 +16,8 @@
 
 from __future__ import annotations
 
-from collections.abc import Callable, Sequence
+from collections.abc import Callable
 from functools import partial
-from types import EllipsisType
 from typing import Any
 import warnings
 
@@ -28,7 +27,6 @@ from jax._src import config
 from jax._src import core
 from jax._src import dtypes
 from jax._src import numpy as jnp
-from jax._src import sharding
 from jax._src import tree_util
 from jax._src import util
 from jax._src.lax import lax
@@ -37,19 +35,15 @@ from jax._src.numpy import indexing
 from jax._src.numpy import reductions
 from jax._src.numpy.util import check_arraylike, promote_dtypes
 from jax._src.pjit import auto_axes
-from jax._src.sharding_impls import canonicalize_sharding
-from jax._src.typing import Array, ArrayLike
+from jax._src.sharding_impls import NamedSharding
+from jax._src.typing import Array, ArrayLike, Index
 
 
-SingleIndex = int | slice | Sequence[int] | Array | EllipsisType | None
-Index = SingleIndex | tuple[SingleIndex, ...]
-Scalar = complex | float | int | np.number
-
-
-def _scatter_update(x: ArrayLike, idx: Index, y: ArrayLike, scatter_op: Callable[..., Array],
+def _scatter_update(x: ArrayLike, idx: Index | tuple[Index, ...],
+                    y: ArrayLike, scatter_op: Callable[..., Array],
                     indices_are_sorted: bool, unique_indices: bool,
                     mode: slicing.GatherScatterMode | str | None = None, normalize_indices: bool = True,
-                    out_sharding: sharding.Sharding | None = None):
+                    out_sharding: NamedSharding | None = None):
   """Helper for indexed updates.
 
   Computes the value of x that would result from computing::
@@ -79,27 +73,27 @@ def _scatter_update(x: ArrayLike, idx: Index, y: ArrayLike, scatter_op: Callable
 
   # XLA gathers and scatters are very similar in structure; the scatter logic
   # is more or less a transpose of the gather equivalent.
-  treedef, static_idx, dynamic_idx = indexing.split_index_for_jit(idx, x.shape)
+  indexer = indexing.NDIndexer.from_raw_indices(idx, x.shape).expand_bool_indices()
+  dynamic_idx, treedef = tree_util.tree_flatten(indexer)
 
   internal_scatter = partial(
       _scatter_impl, scatter_op=scatter_op, treedef=treedef,
-      static_idx=static_idx, indices_are_sorted=indices_are_sorted,
+      indices_are_sorted=indices_are_sorted,
       unique_indices=unique_indices, mode=mode,
       normalize_indices=normalize_indices)
   if out_sharding is not None:
-    out_sharding = canonicalize_sharding(out_sharding, 'scatter')  # type: ignore
     return auto_axes(internal_scatter, out_sharding=out_sharding,
                      axes=out_sharding.mesh.explicit_axes  # type: ignore
                      )(x, y, dynamic_idx)
-  return internal_scatter(x, y, dynamic_idx)
+  return internal_scatter(x, y, tuple(dynamic_idx))
 
 
 # TODO(phawkins): re-enable jit after fixing excessive recompilation for
 # slice indexes (e.g., slice(0, 5, None), slice(10, 15, None), etc.).
-# @partial(jit, static_argnums=(2, 3, 4))
+# @jit(static_argnums=(2, 3, 4))
 def _scatter_impl(x: ArrayLike, y: ArrayLike, dynamic_idx: tuple[Any, ...], *,
                   scatter_op: Callable[..., Array],
-                  treedef: tree_util.PyTreeDef, static_idx: tuple[Any, ...],
+                  treedef: tree_util.PyTreeDef,
                   indices_are_sorted: bool, unique_indices: bool,
                   mode: slicing.GatherScatterMode | str | None, normalize_indices: bool):
   dtype = lax.dtype(x)
@@ -110,13 +104,12 @@ def _scatter_impl(x: ArrayLike, y: ArrayLike, dynamic_idx: tuple[Any, ...], *,
     warnings.warn(
       "scatter inputs have incompatible types: cannot safely cast value "
       f"from dtype={lax.dtype(y)} to dtype={lax.dtype(x)} with "
-      f"jax_numpy_dtype_promotion={config.numpy_dtype_promotion.value!r}. "
+      f"jax_numpy_dtype_promotion={config.numpy_dtype_promotion.value}. "
       "In future JAX releases this will result in an error.",
       FutureWarning)
 
-  idx = indexing.merge_static_and_dynamic_indices(treedef, static_idx, dynamic_idx)
-  indexer = indexing.index_to_gather(np.shape(x), idx,
-                                     normalize_indices=normalize_indices)
+  general_indexer = tree_util.tree_unflatten(treedef, dynamic_idx)
+  indexer = general_indexer.to_gather(core.typeof(x).sharding, normalize_indices=normalize_indices)
 
   # Avoid calling scatter if the slice shape is empty, both as a fast path and
   # to handle cases like zeros(0)[array([], int32)].
@@ -126,7 +119,8 @@ def _scatter_impl(x: ArrayLike, y: ArrayLike, dynamic_idx: tuple[Any, ...], *,
   x, y = promote_dtypes(x, y)
 
   # Broadcast `y` to the slice output shape.
-  y = jnp.broadcast_to(y, tuple(indexer.slice_shape))
+  y = jnp.broadcast_to(y, tuple(indexer.slice_shape),
+                       out_sharding=indexer.slice_sharding)
   # Collapse any `None`/`np.newaxis` dimensions.
   y = jnp.squeeze(y, axis=indexer.newaxis_dims)
   if indexer.reversed_y_dims:
