@@ -18,23 +18,23 @@ from __future__ import annotations
 
 from collections.abc import Sequence
 import dataclasses
-import os
-import tempfile
+import json
 from typing import cast
 
 import jax
 from jax import dtypes
-from jax._src import config
 from jax._src import core as jax_core
 from jax._src import frozen_dict
 from jax._src import sharding_impls
 from jax._src import tpu_custom_call
 from jax._src.interpreters import mlir
 from jax._src.lib.mlir import ir
+from jax._src.lib.mlir import passmanager
 from jax._src.pallas import core as pallas_core
 from jax._src.pallas.mosaic import core as tpu_core
 from jax._src.pallas.mosaic import lowering
-from jax._src.pallas.mosaic import verification
+from jax._src.pallas.mosaic import sc_lowering
+from jax._src.state import types as state_types
 from jax.experimental import mosaic
 from jax.experimental.mosaic.dialects import tpu
 
@@ -47,7 +47,7 @@ def _maybe_cast_to_int(x: jax.Array | jax_core.AbstractValue):
   after loading from a memref inside of the kernel.
   """
   assert isinstance(
-      x, (jax.Array, jax_core.ShapedArray, jax_core.DShapedArray)
+      x, (jax.Array, jax_core.ShapedArray, state_types.AbstractLinVal)
   ), type(x)
   if isinstance(x, jax.Array):
     if dtypes.issubdtype(x.dtype, jax.numpy.bool_):
@@ -55,25 +55,17 @@ def _maybe_cast_to_int(x: jax.Array | jax_core.AbstractValue):
     return x
   else:
     if dtypes.issubdtype(x.dtype, jax.numpy.bool_):
+      if isinstance(x, state_types.AbstractLinVal):
+        raise NotImplementedError  # TODO(mattjj,sharadmv)
       return jax_core.ShapedArray(x.shape, lowering.BOOL_MEMREF_TYPE)
     return x
 
-_DUMP_PROMELA_TO = config.string_flag(
-    "jax_pallas_dump_promela_to",
-    default=os.getenv("JAX_PALLAS_DUMP_PROMELA_TO", ""),
-    help=(
-        "If set, dumps a Promela model of the kernel to the specified"
-        " directory. The model can verify that the kernel is free of data"
-        " races, deadlocks, etc."
-    ),
-)
-
 
 def _get_memory_space_from_aval(
-    out_aval: jax_core.AbstractValue,
+    out_aval: jax_core.AbstractValue, kernel_type: tpu_core.KernelType
 ) -> tpu_custom_call.MemorySpace | None:
   if not isinstance(out_aval, jax_core.ShapedArray):
-    raise ValueError('Memory spaces not defined for non-ShapedArrays')
+    raise ValueError("Memory spaces not defined for non-ShapedArrays")
   if not isinstance(out_aval, pallas_core.ShapedArrayWithMemorySpace):
     # If we are passed a regular old ShapedArray, we don't constrain the
     # memory space
@@ -81,10 +73,6 @@ def _get_memory_space_from_aval(
   # If we are passed an aval with an explicit memory space tag, we use it
   # to constrain the memory space.
   match out_aval.memory_space:
-    case None:
-      return None
-    case tpu_core.MemorySpace.ANY:
-      return None
     case tpu_core.MemorySpace.HBM:
       return tpu_custom_call.MemorySpace.HBM
     case tpu_core.MemorySpace.VMEM:
@@ -92,18 +80,29 @@ def _get_memory_space_from_aval(
     case tpu_core.MemorySpace.SMEM:
       return tpu_custom_call.MemorySpace.SMEM
     case tpu_core.MemorySpace.SEMAPHORE:
-      return tpu_custom_call.MemorySpace.SEMAPHORE_MEM
+      match kernel_type:
+        case tpu_core.KernelType.SC_SCALAR_SUBCORE:
+          return tpu_custom_call.MemorySpace.SC_SCALAR_SEMAPHORE_MEM
+        case tpu_core.KernelType.TC:
+          return tpu_custom_call.MemorySpace.SEMAPHORE_MEM
+        case _:
+          raise ValueError(f"Invalid kernel type for semaphore: {kernel_type}")
+    case tpu_core.MemorySpace.HOST:
+      return tpu_custom_call.MemorySpace.HOST
   return None
 
 
 def _get_memory_spaces_from_avals(
-    avals: Sequence[jax_core.AbstractValue],
+    avals: Sequence[jax_core.AbstractValue], kernel_type: tpu_core.KernelType
 ) -> tuple[tpu_custom_call.MemorySpace | None, ...] | None:
   memory_spaces = None
   if any(
       isinstance(aval, pallas_core.ShapedArrayWithMemorySpace) for aval in avals
   ):
-    memory_spaces = tuple(map(_get_memory_space_from_aval, avals))
+    memory_spaces = tuple(
+        _get_memory_space_from_aval(aval, kernel_type=kernel_type)
+        for aval in avals
+    )
   return memory_spaces
 
 
@@ -120,22 +119,22 @@ def pallas_call_tpu_lowering_rule(
     cost_estimate: pallas_core.CostEstimate | None,
     out_avals: tuple[jax_core.AbstractValue, ...],
     metadata: frozen_dict.FrozenDict[str, str] | None,
+    name: str | None,
 ):
   """Lowers a pallas_call to a Mosaic TPU custom call."""
-  del mesh, interpret  # Unused.
+  del interpret  # Unused.
 
-  debug_info = jaxpr._debug_info
+  debug_info = jaxpr.debug_info
   if debug:
     print(f"\nThe kernel jaxpr for pallas_call {debug_info.func_src_info}:")
     print(jaxpr)
 
   if "mosaic_tpu" in compiler_params:
-    mosaic_params = cast(
-        tpu_core.CompilerParams, compiler_params["mosaic_tpu"]
-    )
+    mosaic_params = cast(tpu_core.CompilerParams, compiler_params["mosaic_tpu"])
   else:
     mosaic_params = tpu_core.CompilerParams()
 
+  del mesh
   jax_mesh = None
   axis_context = ctx.module_context.axis_context
   if axis_context is not None:
@@ -146,63 +145,38 @@ def pallas_call_tpu_lowering_rule(
   mlir_ctx.load_all_available_dialects()
   tpu.register_dialect(mlir_ctx)
 
-  def lower_module(for_verification: bool):
-    if for_verification:
-      mlir_ctx.allow_unregistered_dialects = True
-    with mlir_ctx, ir.Location.unknown(mlir_ctx):
-      return lowering.lower_jaxpr_to_module(
-          ctx,
-          grid_mapping,
-          jaxpr,
-          dimension_semantics=mosaic_params.dimension_semantics,
-          kernel_type=mosaic_params.kernel_type,
-          mesh=jax_mesh,
-          for_verification=for_verification,
-          dynamic_shape_replacement_enabled=pallas_core.dynamic_shapes_export_enabled(),
+  match (kernel_type := mosaic_params.kernel_type):
+    case tpu_core.KernelType.TC:
+      lower_jaxpr_to_module = lowering.lower_jaxpr_to_module
+    case tpu_core.KernelType.SC_SCALAR_SUBCORE | tpu_core.KernelType.SC_VECTOR_SUBCORE:
+      lower_jaxpr_to_module = sc_lowering.lower_jaxpr_to_module
+    case _:
+      raise ValueError(
+          f"Unsupported kernel type: {mosaic_params.kernel_type}"
       )
 
-  mosaic_module, extra_args = lower_module(for_verification=False)
+  with mlir_ctx, ir.Location.unknown(mlir_ctx):
+    mosaic_module = lower_jaxpr_to_module(
+        ctx,
+        grid_mapping,
+        jaxpr,
+        dimension_semantics=mosaic_params.dimension_semantics,
+        kernel_type=kernel_type,
+        mesh=jax_mesh,
+        dynamic_shape_replacement_enabled=pallas_core.dynamic_shapes_export_enabled(),
+    )
+
   if debug:
+    pm = passmanager.PassManager.parse("builtin.module(canonicalize)", mlir_ctx)
+    pm.run(mosaic_module.operation)
     print(f"\nThe Mosaic module for pallas_call {debug_info.func_src_info}:")
     print(mosaic_module)
-  num_extra_args = len(extra_args)
   num_dyn_bounds = grid_mapping.num_dynamic_grid_bounds
   input_output_aliases = tuple(
-      (a[0] + num_dyn_bounds + num_extra_args, a[1])
+      (a[0] + num_dyn_bounds, a[1])
       for a in input_output_aliases
   )
 
-  if promela_dump_path := _DUMP_PROMELA_TO.value:
-    num_devices = 1 if jax_mesh is None else jax_mesh.devices.size
-    num_cores = (
-        jax.devices()[0].num_cores
-        if jax_mesh is None
-        else jax_mesh.devices[0].num_cores
-    )
-    verification_module, _ = lower_module(for_verification=True)
-    model = verification.export_promela_model(
-        verification_module, num_devices, num_cores
-    )
-    if promela_dump_path == "stdout":
-      print(f"The Promela model for pallas_call {debug_info.func_src_info}:")
-      print(model)
-    else:
-      if promela_dump_path == "sponge":
-        promela_dump_path = os.getenv("TEST_UNDECLARED_OUTPUTS_DIR", "")
-        if not promela_dump_path:
-          raise ValueError(
-              "TEST_UNDECLARED_OUTPUTS_DIR must be set when"
-              " --jax_pallas_dump_promela_to=sponge"
-          )
-      dump_ctx = tempfile.NamedTemporaryFile(
-          mode="w",
-          prefix=mlir.sanitize_name(debug_info.func_name) + "-",
-          suffix=".pml",
-          dir=promela_dump_path, delete=False,
-      )
-      with dump_ctx as f:
-        f.write(model)
-
   # Replace in_avals to physical avals.
   # This step is required for mapping logical types to physical types.
   # (e.g. PRNG key -> uint32[2])
@@ -213,27 +187,26 @@ def pallas_call_tpu_lowering_rule(
   def _maybe_cast_inputs(*args):
     args = [_maybe_cast_to_int(x) for x in args]
     return args
+
   kernel_in_avals = [_maybe_cast_to_int(x) for x in ctx.avals_in]
-  kernel_out_avals = [_maybe_cast_to_int(x) for x in out_avals]
+  kernel_out_avals = [_maybe_cast_to_int(x) for x in ctx.avals_out]
   cast_ctx = ctx.replace(avals_out=kernel_in_avals)
   in_nodes = mlir.lower_fun(_maybe_cast_inputs)(cast_ctx, *in_nodes)
 
   # Dynamic grid bounds have to go at the front.
   dynamic_grid_args, args = in_nodes[:num_dyn_bounds], in_nodes[num_dyn_bounds:]
   kernel_ctx = ctx.replace(avals_in=kernel_in_avals, avals_out=kernel_out_avals)
-  output_memory_spaces = _get_memory_spaces_from_avals(out_avals)
+  output_memory_spaces = _get_memory_spaces_from_avals(
+      out_avals, kernel_type=kernel_type
+  )
   input_memory_spaces = None
   if any(
       isinstance(aval, pallas_core.ShapedArrayWithMemorySpace)
       for aval in ctx.avals_in
   ):
-    # TODO(sharadmv): Support dynamic grid bounds and extra args.
-    if num_dyn_bounds != 0 or len(extra_args) > 0:
-      raise NotImplementedError(
-          "Dynamic grid bounds and extra args are not supported when"
-          " specifying memory spaces for inputs."
-      )
-    input_memory_spaces = _get_memory_spaces_from_avals(ctx.avals_in)
+    input_memory_spaces = _get_memory_spaces_from_avals(
+        ctx.avals_in, kernel_type=kernel_type
+    )
   if cost_estimate is not None:
     mosaic_cost_estimate = cast(
         tpu_custom_call.CostEstimate, dataclasses.asdict(cost_estimate)
@@ -249,14 +222,74 @@ def pallas_call_tpu_lowering_rule(
           input_output_alias[1]
       ]
     input_memory_spaces = tuple(input_memory_spaces_list)
+  if input_memory_spaces is not None:
+    # Filter out the memory spaces that are not supported for input memory
+    # spaces.
+    input_memory_spaces = tuple(
+        i
+        if i
+        in {  # pylint: disable=g-long-ternary
+            tpu_custom_call.MemorySpace.HBM,
+            tpu_custom_call.MemorySpace.VMEM,
+            tpu_custom_call.MemorySpace.SMEM,
+        }
+        else None
+        for i in input_memory_spaces
+    )
+  has_side_effects: bool | tpu_custom_call.TpuSideEffectType
+  match mosaic_params.has_side_effects:
+    case bool():
+      has_side_effects = mosaic_params.has_side_effects
+    case tpu_core.SideEffectType.PURE:
+      has_side_effects = tpu_custom_call.TpuSideEffectType.PURE
+    case tpu_core.SideEffectType.DATAFLOW_SIDE_EFFECTING:
+      has_side_effects = (
+          tpu_custom_call.TpuSideEffectType.DATAFLOW_SIDE_EFFECTING
+      )
+    case tpu_core.SideEffectType.SIDE_EFFECTING:
+      has_side_effects = tpu_custom_call.TpuSideEffectType.SIDE_EFFECTING
+    case _:
+      raise ValueError(f"Invalid side effect type: {mosaic_params.has_side_effects}")
+  tiling: tpu_custom_call.Tiling | None = None
+  if mosaic_params.use_tc_tiling_on_sc is not None:
+    if kernel_type not in (
+        tpu_core.KernelType.SC_SCALAR_SUBCORE,
+        tpu_core.KernelType.SC_VECTOR_SUBCORE,
+    ):
+      raise ValueError(
+          "use_tc_tiling_on_sc= is only supported for SC_*_SUBCORE kernels"
+      )
+
+    tiling = (
+        tpu_custom_call.Tiling.COMPACT
+        if mosaic_params.use_tc_tiling_on_sc
+        else tpu_custom_call.Tiling.SPARSE_CORE
+    )
+  dict_metadata = dict(metadata) if metadata is not None else {}
+  del metadata
+  if jax_mesh is not None:
+    mesh_axes = {
+        e.name
+        for e in jaxpr.effects
+        if isinstance(e, jax_core.NamedAxisEffect)
+        # Filter for only device mesh axis name effects
+        and e.name in jax_mesh.axis_names
+    }
+    # Only put mesh axes in metadata if there are any.
+    if mesh_axes:
+      if "mesh_axes" in dict_metadata:
+        raise ValueError("Metadata already contains mesh axes.")
+      mesh_axes_list = list(mesh_axes)
+      if all(isinstance(a, str) for a in mesh_axes):
+        mesh_axes_list = sorted(mesh_axes)  # type: ignore
+      dict_metadata["mesh_axes"] = json.dumps(mesh_axes_list)
   out_nodes = mosaic.lower_module_to_custom_call(
       kernel_ctx,
       *dynamic_grid_args,
-      *extra_args,
       *args,
       module=mosaic_module,
       out_type=kernel_out_avals,
-      kernel_name=mlir.sanitize_name(debug_info.func_name),
+      kernel_name=mlir.sanitize_name(name or debug_info.func_name),
       cost_estimate=mosaic_cost_estimate,
       vmem_limit_bytes=mosaic_params.vmem_limit_bytes,
       flags=mosaic_params.flags,
@@ -265,16 +298,25 @@ def pallas_call_tpu_lowering_rule(
       serialization_format=mosaic_params.serialization_format,
       internal_scratch_in_bytes=mosaic_params.internal_scratch_in_bytes,
       collective_id=mosaic_params.collective_id,
-      has_side_effects=mosaic_params.has_side_effects,
+      has_side_effects=has_side_effects,
       output_memory_spaces=output_memory_spaces,
       disable_bounds_checks=mosaic_params.disable_bounds_checks,
       input_memory_spaces=input_memory_spaces,
-      metadata=dict(metadata) if metadata is not None else None,
+      metadata=dict_metadata,
+      skip_device_barrier=mosaic_params.skip_device_barrier,
+      allow_collective_id_without_custom_barrier=mosaic_params.allow_collective_id_without_custom_barrier,
+      shape_invariant_numerics=mosaic_params.shape_invariant_numerics,
+      tiling=tiling,
   )
-  _maybe_cast_to_bool = lambda x, aval: x.astype(
-      jax.numpy.bool_) if aval.dtype == jax.numpy.bool_ else x
+  _maybe_cast_to_bool = (
+      lambda x, aval: x.astype(jax.numpy.bool_)
+      if aval.dtype == jax.numpy.bool_
+      else x
+  )
+
   def _maybe_cast_outputs(*args):
     args = [_maybe_cast_to_bool(x, aval) for x, aval in zip(args, out_avals)]
     return args
+
   cast_ctx = ctx.replace(avals_in=kernel_out_avals)
   return mlir.lower_fun(_maybe_cast_outputs)(cast_ctx, *out_nodes)
