@@ -23,6 +23,7 @@ from typing import Any
 import jax
 from jax._src import core as jax_core
 from jax._src import dtypes
+from jax._src import effects
 from jax._src import pretty_printer as pp
 from jax._src import prng as jax_prng
 from jax._src import random as jax_random
@@ -989,9 +990,16 @@ prng_seed_p = jax_core.Primitive("prng_seed")
 prng_seed_p.multiple_results = True
 
 
-@prng_seed_p.def_abstract_eval
+class PRNGEffect(effects.Effect):
+  pass
+prng_effect = PRNGEffect()
+effects.control_flow_allowed_effects.add_type(PRNGEffect)
+pl_core.kernel_local_effects.add_type(PRNGEffect)
+
+
+@prng_seed_p.def_effectful_abstract_eval
 def _prng_seed_abstract_eval(*_):
-  return []
+  return [], {prng_effect}
 
 
 def prng_seed(*seeds: int | jax.Array) -> None:
@@ -1007,7 +1015,6 @@ def prng_seed(*seeds: int | jax.Array) -> None:
 prng_random_bits_p = jax_core.Primitive(
     'prng_random_bits')
 
-
 @prng_random_bits_p.def_abstract_eval
 def _prng_random_bits_abstract_eval(*, shape):
   return jax_core.ShapedArray(shape, jnp.dtype("int32"))
@@ -1079,6 +1086,7 @@ def _stochastic_round_abstract_eval(x, random_bits, *, target_dtype):
     )
   return jax_core.ShapedArray(x.shape, target_dtype)
 
+
 def _get_elementwise_packing_factor(unpacked_dtype, packed_dtype):
   unpacked_bitwidth = dtypes.itemsize_bits(unpacked_dtype)
   packed_bitwidth = dtypes.itemsize_bits(packed_dtype)
@@ -1105,13 +1113,22 @@ def _pack_elementwise_abstract_eval(*xs, packed_dtype):
     raise ValueError("All sources must have the same shape")
   if not all(x.dtype == first.dtype for x in xs):
     raise ValueError("All sources must have the same dtype")
+  if not (first.dtype == jnp.float32 and packed_dtype == jnp.bfloat16) and not (
+      jnp.issubdtype(first.dtype, jnp.integer)
+      and jnp.issubdtype(packed_dtype, jnp.integer)
+  ):
+    raise ValueError(
+        "Only f32 -> bf16 and int -> int are supported. Got"
+        f" {first.dtype} and {packed_dtype}"
+    )
   packing_factor = _get_elementwise_packing_factor(first.dtype, packed_dtype)
   if len(xs) != packing_factor:
     raise ValueError(
         "The number of sources must match the packing factor "
         f"({packing_factor}), got {len(xs)}"
     )
-  return jax_core.ShapedArray(first.shape, jnp.uint32)
+  out_dtype = jnp.dtype(f"uint{dtypes.itemsize_bits(first.dtype)}")
+  return jax_core.ShapedArray(first.shape, out_dtype)
 
 
 unpack_elementwise_p = jax_core.Primitive("unpack_elementwise")
@@ -1214,3 +1231,46 @@ def touch(ref: jax.Array | state.TransformedRef) -> None:
 @touch_p.def_effectful_abstract_eval
 def _touch_abstract_eval(ref: jax.Array):
   return [], {state.ReadEffect(0), state.WriteEffect(0)}
+
+
+trace_value_p = jax_core.Primitive("trace_value")
+trace_value_p.multiple_results = True
+
+
+def trace_value(label: str, value: jax.Array) -> None:
+  """Emit a scalar value to the current xprof trace scope.
+
+  This appends a dynamic scalar value to the enclosing trace region.
+  The value will appear in xprof trace viewer associated with the trace event.
+
+  Args:
+    label: A string label for this value in xprof.
+    value: A scalar i32 or f32 value to emit.
+
+  Example:
+    # Inside a Pallas kernel:
+    x  = jnp.sum(y > 0)
+    pltpu.trace_value("my_x", x)
+  """
+  trace_value_p.bind(value, label=label)
+
+
+class TraceEffect(effects.Effect):
+  pass
+
+
+trace_effect = TraceEffect()
+effects.control_flow_allowed_effects.add_type(TraceEffect)
+pl_core.kernel_local_effects.add_type(TraceEffect)
+
+
+@trace_value_p.def_effectful_abstract_eval
+def _trace_value_abstract_eval(value, *, label):
+  del label
+  if value.shape:
+    raise ValueError(
+        f"trace_value requires a scalar value, got shape {value.shape}"
+    )
+  if value.dtype not in (jnp.int32, jnp.float32):
+    raise ValueError(f"trace_value requires i32 or f32, got {value.dtype}")
+  return [], {trace_effect}
