@@ -36,7 +36,6 @@ from typing import Union
 
 import numpy as np
 
-from jax._src import config
 from jax._src import util
 from jax._src.lib import pmap_lib
 
@@ -97,7 +96,7 @@ def _sharding_spec_indices(self, shape: tuple[int, ...]) -> np.ndarray:
   # is used to extract the corresponding shard of the logical array.
   shard_indices = np.empty([math.prod(shard_indices_shape)], dtype=np.object_)
   for i, idxs in enumerate(itertools.product(*axis_indices)):
-    shard_indices[i] = idxs  # type: ignore  # numpy 2.2
+    shard_indices[i] = idxs
   shard_indices = shard_indices.reshape(shard_indices_shape)
 
   # Ensure that each sharded axis is used exactly once in the mesh mapping
@@ -172,12 +171,8 @@ def pmap_sharding_spec(nrep, axis_size, sharded_shape: Sequence[int],
       return a
     # replication_factor represents the product of inner pmaps, so it goes
     # after the outer pmapped axis at index 0
-    if config.pmap_no_rank_reduction.value:
-      sharding = util.tuple_update(
-          pspec.sharding, map_axis, Chunked([axis_size]))
-    else:
-      sharding = util.tuple_insert(
-          pspec.sharding, map_axis, Unstacked(axis_size))
+    sharding = util.tuple_update(
+        pspec.sharding, map_axis, Chunked([axis_size]))
     return ShardingSpec(
       sharding=sharding,
       mesh_mapping=itertools.chain(
@@ -192,10 +187,7 @@ def pmap_sharding_spec(nrep, axis_size, sharded_shape: Sequence[int],
 def create_pmap_sharding_spec(shape: tuple[int, ...], sharded_dim: int = 0,
                               sharded_dim_size: int | None = None):
   if sharded_dim is not None:
-    if config.pmap_no_rank_reduction.value:
-      sharded_shape = util.tuple_update(shape, sharded_dim, 1)
-    else:
-      sharded_shape = util.tuple_delete(shape, sharded_dim)
+    sharded_shape = util.tuple_update(shape, sharded_dim, 1)
     if sharded_dim_size is None:
       sharded_dim_size = shape[sharded_dim]
   else:
