@@ -76,7 +76,10 @@ class AccumEffect(RefEffect):
   name: str = "Accum"
 
 effects.control_flow_allowed_effects.add_type(RefEffect)
+effects.custom_derivatives_allowed_effects.add_type(RefEffect)
+effects.custom_derivatives_allowed_effects.add_type(core.InternalMutableArrayEffect)
 effects.partial_eval_kept_effects.add_type(RefEffect)
+effects.remat_allowed_effects.add_type(RefEffect)
 
 StateEffect = Union[ReadEffect, WriteEffect, AccumEffect]
 
@@ -144,6 +147,18 @@ class RefReshaper:
       shape = shape[0]
     if not shape:
       raise ValueError("Cannot reshape ref to empty shape")
+    if any(s == -1 for s in shape):
+      num_elements = math.prod(ref_or_view.shape)
+      defined_dims = [d for d in shape if d != -1]
+      if len(defined_dims) != len(shape) - 1:
+        raise ValueError(f"At most one dimension can be -1, but got {shape}")
+      if num_elements % math.prod(defined_dims):
+        raise ValueError(
+            f"Specified dims {shape} do not evenly divide the size of the "
+            f"ref ({num_elements})."
+        )
+      remaining_dim = num_elements // math.prod(defined_dims)
+      shape = tuple(d if d != -1 else remaining_dim for d in shape)
     if np.prod(shape) != np.prod(ref_or_view.shape):
       raise TypeError(
           f"cannot reshape ref of shape {ref_or_view.shape} into shape {shape}"
@@ -174,7 +189,7 @@ class RefReshaper:
     del shape  # Unused
     return self.shape
 
-  def transform_dtype(self, dtype):
+  def transform_dtype(self, dtype: DTypeLike | None) -> DTypeLike | None:
     del dtype  # Unused
     return self.dtype
 
@@ -189,6 +204,45 @@ class RefReshaper:
     return pp.text(f"{{reshape({self.dtype}{list(self.shape)})}}")
 
 
+@tree_util.register_dataclass
+@dataclasses.dataclass(frozen=True)
+class RefTransposer:
+  permutation: tuple[int, ...] = dataclasses.field(metadata=dict(static=True))
+
+  @classmethod
+  def from_ref_new_permutation(
+      cls, ref_or_view: Any, *perm: int
+  ) -> RefTransposer:
+    if len(perm) == 1 and isinstance(perm[0], tuple):
+      perm = perm[0]
+    if len(perm) != ref_or_view.ndim:
+      raise ValueError(
+          f"Permutation {perm} does not match the rank of the ref"
+          f" ({ref_or_view.ndim})"
+      )
+    return cls(perm)
+
+  def transform_shape(
+      self, shape: tuple[int | Array, ...] | None
+  ) -> tuple[int | Array, ...] | None:
+    if shape is None:
+      return None
+    return tuple(shape[i] for i in self.permutation)
+
+  def transform_dtype(self, dtype):
+    return dtype
+
+  def transform_sharding(self, sharding):
+    # If there are no explicit axes, do nothing.
+    if all(p is None for p in sharding.spec):
+      return sharding
+    raise NotImplementedError
+
+  def pretty_print(self, context: core.JaxprPpContext) -> pp.Doc:
+    del context  # Unused.
+    return pp.text(f"{{transpose({list(self.permutation)})}}")
+
+
 class Transform(Protocol):
 
   def transform_shape(
@@ -201,9 +255,7 @@ class Transform(Protocol):
     """
     return shape
 
-  def transform_dtype(
-      self, dtype: DTypeLike | None
-  ) -> DTypeLike | None:
+  def transform_dtype(self, dtype: DTypeLike | None) -> DTypeLike | None:
     """Transform the dtype.
 
     Can return None if the input dtype is not known, but must return a concrete
@@ -281,6 +333,7 @@ class TransformedRef:
 
   ndim = property(lambda self: len(self.shape))
   size = property(lambda self: math.prod(self.shape))
+  T = property(lambda self: self.transpose(*reversed(range(self.ndim))))
 
   @property
   def at(self) -> RefIndexer:
@@ -298,6 +351,10 @@ class TransformedRef:
         (*self.transforms, RefReshaper.from_ref_new_shape(self, *shape)),
     )
 
+  def transpose(self, *permutation):
+    transposer = RefTransposer.from_ref_new_permutation(self, *permutation)
+    return TransformedRef(self.ref, (*self.transforms, transposer))
+
   def set(self, value, idx=()):
     from jax._src.state.primitives import ref_set  # pytype: disable=import-error
     return ref_set(self, idx, value)
@@ -322,13 +379,51 @@ class TransformedRef:
     return ref_set(self, slc, value)
 
 
+def get_transforms_shape(
+    ts: Sequence[Transform], shape: tuple[int | Array, ...]
+) -> tuple[int | Array, ...]:
+  for t in ts:
+    shape = t.transform_shape(shape)  # type: ignore
+  assert shape is not None
+  return shape
+
+
 # We need an aval for `Ref`s so we can represent `get` and `swap` in Jaxprs.
 class AbstractRef(core.AbstractValue):
-  __slots__ = ["inner_aval", "memory_space"]
+  """Abstract mutable array reference.
 
-  def __init__(self, inner_aval: core.AbstractValue, memory_space: Any = None):
+  Refer to the `Ref guide`_ for more information.
+
+  .. _Ref guide: https://docs.jax.dev/en/latest/array_refs.html
+  """
+  __slots__ = ["inner_aval", "memory_space", "kind"]
+
+  def __init__(self, inner_aval: core.AbstractValue, memory_space: Any = None,
+               kind: Any = None):
     self.inner_aval = inner_aval
     self.memory_space = memory_space
+    self.kind = kind
+
+  @property
+  def is_high(self):
+    return self.inner_aval.is_high
+
+  def lo_ty(self):
+    return [
+        AbstractRef(x, memory_space=self.memory_space)
+        for x in self.inner_aval.lo_ty()
+    ]
+
+  def lower_val(self, ref):
+    if not self.is_high:
+      return [ref]
+    return self.inner_aval.lower_val(ref._refs)  # type: ignore
+
+  def raise_val(self, *vals):
+    if not self.is_high:
+      ref, = vals
+      return ref
+    return core.Ref(self, self.inner_aval.raise_val(*vals))  # type: ignore
 
   @property
   def weak_type(self) -> bool:
@@ -339,10 +434,11 @@ class AbstractRef(core.AbstractValue):
   def update_weak_type(self, weak_type):
     return self.update(inner_aval=self.inner_aval.update_weak_type(weak_type))
 
-  def update(self, inner_aval=None, memory_space=None):
+  def update(self, inner_aval=None, memory_space=None, kind=None):
     inner_aval = self.inner_aval if inner_aval is None else inner_aval
     memory_space = self.memory_space if memory_space is None else memory_space
-    return AbstractRef(inner_aval, memory_space)
+    kind = self.kind if kind is None else kind
+    return AbstractRef(inner_aval, memory_space, kind)
 
   ndim = property(lambda self: len(self.shape))
   size = property(lambda self: math.prod(self.shape))
@@ -395,11 +491,19 @@ class AbstractRef(core.AbstractValue):
 
   @core.aval_method
   def bitcast(self, dtype):
-    return TransformedRef(self, (RefBitcaster.from_ref_new_dtype(self, dtype),))
+    return TransformedRef(self, ()).bitcast(dtype)
 
   @core.aval_method
   def reshape(self, *shape):
-    return TransformedRef(self, (RefReshaper.from_ref_new_shape(self, *shape),))
+    return TransformedRef(self, ()).reshape(*shape)
+
+  @core.aval_method
+  def transpose(self, *permutation):
+    return TransformedRef(self, ()).transpose(*permutation)
+
+  @core.aval_property
+  def T(self):
+    return TransformedRef(self, ()).T
 
   @core.aval_method
   @staticmethod
@@ -419,6 +523,12 @@ class AbstractRef(core.AbstractValue):
     from jax._src.state.primitives import ref_set  # pytype: disable=import-error
     return ref_set(tracer, idx, value)
 
+  @core.aval_method
+  @staticmethod
+  def addupdate(tracer, value, idx=()):
+    from jax._src.state.primitives import ref_addupdate  # pytype: disable=import-error
+    ref_addupdate(tracer, idx, value)
+
   def _getitem(self, tracer, idx) -> Array:
     from jax._src.state.primitives import ref_get  # pytype: disable=import-error
     return ref_get(tracer, idx)
@@ -427,13 +537,30 @@ class AbstractRef(core.AbstractValue):
     from jax._src.state.primitives import ref_set  # pytype: disable=import-error
     return ref_set(tracer, idx, value)
 
-  def __repr__(self) -> str:
+  def _addupdate(self, tracer, idx, value):
+    from jax._src.state.primitives import ref_addupdate  # pytype: disable=import-error
+    ref_addupdate(tracer, idx, value)
+
+  def str_short(self, short_dtypes=False, mesh_axis_types=False) -> str:
+    inner_aval_str = self.inner_aval.str_short(
+        short_dtypes=short_dtypes,
+        mesh_axis_types=mesh_axis_types,
+    )
     if self.memory_space is not None:
-      return f'Ref<{self.memory_space}>{{{self.inner_aval.str_short()}}}'
-    return f'Ref{{{self.inner_aval.str_short()}}}'
+      return f'Ref<{self.memory_space}>{{{inner_aval_str}}}'
+    return f'Ref{{{inner_aval_str}}}'
+
+  def __repr__(self) -> str:
+    return self.str_short()
+  __str__ = __repr__
 
   def to_tangent_aval(self):
-    return AbstractRef(self.inner_aval.to_tangent_aval(), self.memory_space)
+    return AbstractRef(self.inner_aval.to_tangent_aval(), self.memory_space,
+                       kind=self.kind)
+
+  def to_cotangent_aval(self):
+    return AbstractRef(self.inner_aval.to_cotangent_aval(), self.memory_space,
+                       kind=self.kind)
 
   def __eq__(self, other):
     return (type(self) is type(other) and self.inner_aval == other.inner_aval
@@ -444,12 +571,12 @@ class AbstractRef(core.AbstractValue):
 
 def _map_ref(size, axis, ref_aval):
   return AbstractRef(core.mapped_aval(size, axis, ref_aval.inner_aval),
-                     ref_aval.memory_space)
+                     ref_aval.memory_space, ref_aval.kind)
 
 def _unmap_ref(size, axis, explicit_mesh_axis, ref_aval):
   return AbstractRef(core.unmapped_aval(
       size, axis, ref_aval.inner_aval, explicit_mesh_axis),
-                     ref_aval.memory_space)
+                     ref_aval.memory_space, ref_aval.kind)
 
 core.aval_mapping_handlers[AbstractRef] = (_map_ref, _unmap_ref)
 
@@ -495,3 +622,14 @@ def get_ref_aval_from_value(x: Any):
   if type(x) in _ref_type_aval_mappings:
     return _ref_type_aval_mappings[type(x)](x)
   return _default_value_to_ref_aval(x)
+
+# === pinned, chained LinearVals ===
+
+@dataclasses.dataclass(frozen=True)
+class AbstractLinVal(core.AbstractValue):
+  inner_aval: core.AbstractValue
+  memory_space: Any = None
+
+  shape = property(lambda self: self.inner_aval.shape)  # type: ignore
+  dtype = property(lambda self: self.inner_aval.dtype)  # type: ignore
+  ndim = property(lambda self: self.inner_aval.ndim)  # type: ignore
