@@ -18,7 +18,7 @@ from collections.abc import Callable, Hashable, Iterable, Sequence
 import dataclasses
 import difflib
 import functools
-from functools import partial
+from functools import partial, cached_property
 import operator as op
 import textwrap
 from typing import Any, TypeVar
@@ -93,6 +93,13 @@ def tree_leaves(tree: Any,
   return default_registry.flatten(tree, is_leaf)[0]
 
 
+@export
+def tree_leaves_checked(treedef_expected: PyTreeDef, tree: Any) -> list[Leaf]:
+  flat_vals, treedef_actual = tree_flatten(tree)
+  assert treedef_actual == treedef_expected
+  return flat_vals
+
+
 @export
 def tree_structure(tree: Any,
                    is_leaf: None | (Callable[[Any],
@@ -1094,10 +1101,16 @@ def register_dataclass(
     if not dataclasses.is_dataclass(nodetype):
       raise TypeError("register_dataclass: data_fields and meta_fields are required when"
                       f" nodetype is not a dataclass. Got {nodetype=}.")
-    data_fields = [f.name for f in dataclasses.fields(nodetype)
-                   if not f.metadata.get('static', False)]
-    meta_fields = [f.name for f in dataclasses.fields(nodetype)
-                   if f.metadata.get('static', False)]
+    data_fields = [
+        f.name
+        for f in dataclasses.fields(nodetype)
+        if not f.metadata.get("static", False)
+    ]
+    meta_fields = [
+        f.name
+        for f in dataclasses.fields(nodetype)
+        if f.metadata.get("static", False)
+    ]
 
   assert meta_fields is not None
   assert data_fields is not None
@@ -1125,6 +1138,12 @@ def register_dataclass(
         msg += f" Unexpected fields: {unexpected}."
       raise ValueError(msg)
 
+  if overlap := set(data_fields) & set(meta_fields):
+    raise ValueError(
+        "data_fields and meta_fields must not overlap. Overlapping fields:"
+        f" {overlap}."
+    )
+
   def unflatten_func(meta, data):
     meta_args = tuple(zip(meta_fields, meta))
     data_args = tuple(zip(data_fields, data))
@@ -1353,47 +1372,46 @@ class FlatTree:
        the tuple-returning function would change the tree structure and `unzip`
        wouldn't be able to recover it.
   """
-  def __init__(self, vals:Sequence, treedef:PyTreeDef):
+  # `FlatTree` constructor is private. Use `FlatTree.flatten` instead
+  def __init__(self, vals, treedef: PyTreeDef, statics):
     assert isinstance(treedef, pytree.PyTreeDef)
-    self.tree = treedef
+    if not isinstance(vals, tuple):
+      vals = tuple(vals)
     self.vals = tuple(vals)
+    self.tree = treedef
+    self.statics = statics  # tree-prefix tuple-dict-tree of bools
 
-  def map(self, f:Callable) -> FlatTree:
-    ans_vals = []
-    for x in self.vals:
-      ans_vals.append(f(x))
-    return FlatTree(ans_vals, self.tree)
+  def __eq__(self, other):
+    return (isinstance(other, FlatTree) and self.vals == other.vals
+            and self.tree == other.tree and self.statics == other.statics)
 
-  def map2(self:FlatTree, f:Callable, t2:FlatTree) -> FlatTree:
+  def __hash__(self):
+    return hash((self.vals, self.tree))
 
+  def map(self, f: Callable) -> FlatTree:
+    return self.update(f(x) for x in self.vals)
+
+  def map2(self: FlatTree, f: Callable, t2: FlatTree) -> FlatTree:
     n = len(self)
     assert len(t2) == n
-    ans_vals = []
-    for x1, x2 in zip(self.vals, t2.vals):
-      ans_vals.append(f(x1, x2))
-    return FlatTree(ans_vals, self.tree)
+    return self.update(f(x1, x2) for x1, x2 in zip(self.vals, t2.vals))
 
   def map3(
-      self:FlatTree, f:Callable, t2:FlatTree, t3:FlatTree) -> FlatTree:
+      self: FlatTree, f: Callable, t2: FlatTree, t3: FlatTree) -> FlatTree:
     n = len(self)
     assert len(t2) == n and len(t3) == n
-    ans_vals = []
-    for x1, x2, x3 in zip(self.vals, t2.vals, t3.vals):
-      ans_vals.append(f(x1, x2, x3))
-    return FlatTree(ans_vals, self.tree)
+    return self.update(f(x1, x2, x3)
+                       for x1, x2, x3 in zip(self.vals, t2.vals, t3.vals))
 
-  def zip(self, t2:FlatTree) -> FlatTree:
-    assert False
-
-  def unzip2(self:FlatTree) -> tuple[FlatTree, FlatTree]:
+  def unzip2(self: FlatTree) -> tuple[FlatTree, FlatTree]:
     ys = []
     zs = []
     for y, z in self.vals:
       ys.append(y)
       zs.append(z)
-    return FlatTree(ys, self.tree), FlatTree(zs, self.tree)
+    return self.update(ys), self.update(zs)
 
-  # TODO: add map3, zip3, unzip3 etc. as needed
+  # TODO: add other helpers like map3, zip, unzip3 etc. as needed
 
   @staticmethod
   def pack(tree):
@@ -1404,11 +1422,13 @@ class FlatTree:
     elif isinstance(tree, tuple):
       vals = []
       trees = []
+      staticss = []
       for child_tree in tree:
         child = FlatTree.pack(child_tree)
         vals.extend(child.vals)
         trees.append(child.tree)
-      return FlatTree(vals, treedef_tuple(trees))
+        staticss.append(child.statics)
+      return FlatTree(vals, treedef_tuple(trees), tuple(staticss))
     elif isinstance(tree, dict):
       # only empty case handled for now
       if tree == {}:
@@ -1418,38 +1438,143 @@ class FlatTree:
     else:
       assert False
 
-  def unpack(self:FlatTree) -> tuple[FlatTree, ...]:
+  def unpack(self: FlatTree) -> tuple[FlatTree, ...]:
     # TODO: this is O(N) not O(1) (with N as the number of leaves). If it
     # becomes a problem we can fix it with a fancier data tree.
     trees = treedef_children(self.tree)
     children = []
     offset = 0
-    for tree in trees:
+    for i, tree in enumerate(trees):
+      statics = False if isinstance(self.statics, bool) else self.statics[i]
       new_offset = offset + tree.num_leaves
-      children.append(FlatTree(self.vals[offset:new_offset], tree))
+      children.append(FlatTree(self.vals[offset:new_offset], tree, statics))
       offset = new_offset
     return tuple(children)
 
   @staticmethod
   def flatten(tree: PyTree) -> FlatTree:
-    return FlatTree(*tree_flatten(tree))
+    vals, tree = tree_flatten(tree)
+    return FlatTree(vals, tree, False)
+
+  @staticmethod
+  def flatten_static_argnums(args, static_argnums):
+    if not static_argnums:
+      return FlatTree.flatten(args)
+    else:
+      assert isinstance(args, tuple)
+      num_args = len(args)
+      static_argnums = [i % num_args if i < 0 else i for i in static_argnums]
+      statics = tuple(i in static_argnums for i, _ in enumerate(args))
+      tree_with_statics = tuple(
+          Static(x) if static else x for static, x in zip(statics, args))
+      vals, treedef = tree_flatten(tree_with_statics)
+      return FlatTree(vals, treedef, statics=statics)
+
+  @staticmethod
+  def flatten_static_argnames(kwargs, static_argnames):
+    if not static_argnames:
+      return FlatTree.flatten(kwargs)
+    else:
+      assert isinstance(kwargs, dict)
+      statics = {k : k in static_argnames for k, _ in kwargs.items()}
+      tree_with_statics = {k : Static(v) if statics[k] else v
+                           for k, v in kwargs.items()}
+      vals, treedef = tree_flatten(tree_with_statics)
+      return FlatTree(vals, treedef, statics=statics)
+
+  @staticmethod
+  def flatten_static_argnums_argnames(
+      args, kwargs, static_argnums, static_argnames):
+    return FlatTree.pack((
+        FlatTree.flatten_static_argnums(args, static_argnums),
+        FlatTree.flatten_static_argnames(kwargs, static_argnames)))
 
   def unflatten(self) -> PyTree:
-    return tree_unflatten(self.tree, self.vals)
+    pytree = tree_unflatten(self.tree, self.vals)
+    return unwrap_statics(pytree, self.statics)
 
-  def update_from_list(self, new_vals:list) -> FlatTree:
-    return FlatTree(new_vals, self.tree)
+  @property
+  def tree_without_statics(self):
+    # hardcodes default_registry because it's used implicitly in self.flatten
+    return filter_statics_from_treedef(default_registry, self.tree, self.statics)
+
+  def update(self, new_vals) -> FlatTree:
+    # `new_vals` can be a generator because `FlatTree` forces it to a tuple
+    new = FlatTree(new_vals, self.tree, self.statics)
+    assert len(self.vals) == len(new.vals)
+    return new
+
+  @cached_property
+  def paths(self) -> FlatTree:
+    # TODO(dougalm): find a way to do this without roundtripping
+    try:
+      paths, _ = unzip2(tree_leaves_with_path(self.unflatten()))
+      assert len(paths) == len(self.vals)
+      return self.update(paths)
+    except:
+      return self.update([()] * len(self.vals))  # not our fault
 
   def __len__(self):
+    return self.len
+
+  @cached_property
+  def len(self):
     return self.tree.num_leaves
 
   def __iter__(self):
     return self.vals.__iter__()
 
-  def __eq__(self, other):
-    return (isinstance(other, FlatTree)
-            and self.vals == other.vals
-            and self.tree == other.tree)
+def unwrap_statics(pytree, statics):
+  if statics is False:
+    return pytree
+  elif statics is True:
+    return pytree.val  # pytree should be a `Static` object
+  elif isinstance(pytree, tuple):
+    return tuple(unwrap_statics(p, s) for p, s in zip(pytree, statics))
+  elif isinstance(pytree, dict):
+    return {k : unwrap_statics(p, statics[k]) for k, p in pytree.items()}
+  else:
+    assert False, "unreachable"
 
-  def __hash__(self):
-    return hash((self.vals, self.tree))
+def filter_statics_from_treedef(registry, treedef, statics):
+  if statics is False:
+    return treedef
+  elif statics is True:
+    assert False, "unreachable"
+  elif isinstance(statics, tuple):
+    filtered = tuple(
+        filter_statics_from_treedef(registry, td, s)
+        for td, s in zip(treedef.children(), statics) if s is not True)
+    return treedef.from_node_data_and_children(registry, treedef.node_data(), filtered)  # type: ignore
+  elif isinstance(statics, dict):
+    ty, keys = treedef.node_data()  # type: ignore
+    filtered_keys, filtered_subtrees = unzip2(
+        (k, filter_statics_from_treedef(registry, td, statics[k]))
+        for td, k in zip(treedef.children(), keys) if statics[k] is not True)
+    return treedef.from_node_data_and_children(registry, (ty, filtered_keys), filtered_subtrees)  # type: ignore
+  else:
+    assert False, "unreachable"
+
+@register_static
+@dataclasses.dataclass(frozen=True)
+class Static:
+  val: Any
+
+  def __eq__(self, other):
+    return (type(other) is Static and type(self.val) is type(other.val) and
+            self.val == other.val)
+
+
+def _ensure_inbounds(allow_invalid: bool, num_args: int, argnums: Sequence[int]
+                     ) -> tuple[int, ...]:
+  """Ensure argnum is within bounds. Also resolves negative argnums."""
+  result = []
+  for i in argnums:
+    if i >= num_args and allow_invalid: continue
+    if not -num_args <= i < num_args:
+      raise ValueError(
+          "Positional argument indices, e.g. for `static_argnums`, must have "
+          "value greater than or equal to -len(args) and less than len(args), "
+          f"but got value {i} for len(args) == {num_args}.")
+    result.append(i % num_args)  # Resolve negative
+  return tuple(result)
