@@ -41,6 +41,7 @@ from jax._src import hardware_utils
 from jax._src import traceback_util
 from jax._src import util
 from jax._src.cloud_tpu_init import get_tpu_library_path
+from jax._src.lib import jaxlib_extension_version
 from jax._src.lib import xla_client
 from jax._src.lib import _jax
 from jax._src.lib import _profiler
@@ -108,6 +109,13 @@ _CPU_ENABLE_ASYNC_DISPATCH = config.bool_flag(
     "inline without async dispatch.",
 )
 
+FORCE_DCN_CROSS_HOST_TRANSFERS = config.bool_flag(
+    name="jax_force_dcn_cross_host_transfers",
+    default=False,
+    help="Force cross host transfers to use the DCN socket transfer library "
+         "even when the plugin supports cross-host transfers."
+)
+
 CROSS_HOST_TRANSFER_SOCKET_ADDRESS = config.string_flag(
     name="jax_cross_host_transfer_socket_address",
     default="",
@@ -189,11 +197,19 @@ def make_tpu_client(
     _jax.initialize_pjrt_plugin('tpu')
   if options is None:
     options = {}
+  if jaxlib_extension_version < 397:
+    return _jax.get_c_api_client(
+        "tpu",
+        options,
+        distributed.global_state.client,
+        _make_transfer_server_factory(),
+    )
   return _jax.get_c_api_client(
       "tpu",
       options,
       distributed.global_state.client,
       _make_transfer_server_factory(),
+      FORCE_DCN_CROSS_HOST_TRANSFERS.value,
   )
 
 
@@ -544,11 +560,19 @@ def make_pjrt_c_api_client(
     distribute_options['partition_index'] = partition_index
   if options is not None:
     distribute_options.update(updated_options)
+  if jaxlib_extension_version < 397:
+    return xla_client.make_c_api_client(
+        plugin_name,
+        distribute_options,
+        distributed.global_state.client,
+        _make_transfer_server_factory(),
+    )
   return xla_client.make_c_api_client(
       plugin_name,
       distribute_options,
       distributed.global_state.client,
       _make_transfer_server_factory(),
+      FORCE_DCN_CROSS_HOST_TRANSFERS.value,
   )
 
 
