@@ -35,22 +35,3 @@ from jax._src.ad_util import (
   SymbolicZero as SymbolicZero,
   zero_from_primal as zero_from_primal
 )
-
-_deprecations = {
-    # Finalized for v0.8.0; remove in v0.9.0
-    "custom_jvp_call_jaxpr_p": (
-      ("jax.custom_derivatives.custom_jvp_call_jaxpr_p was deprecated in v0.7.0"
-       " and removed in v0.8.0. use jax.extend.core.primitives.custom_jvp_call_p"
-       " instead, and please note that you must `import jax.extend` explicitly."),
-      None,
-    ),
-}
-
-import typing
-if typing.TYPE_CHECKING:
-  pass
-else:
-  from jax._src.deprecations import deprecation_getattr as _deprecation_getattr
-  __getattr__ = _deprecation_getattr(__name__, _deprecations)
-  del _deprecation_getattr
-del typing
