@@ -13,15 +13,16 @@
 # limitations under the License.
 
 # Note: import <name> as <name> is required for names to be exported.
-# See PEP 484 & https://github.com/google/jax/issues/7570
+# See PEP 484 & https://github.com/jax-ml/jax/issues/7570
 
 from jax._src.dtypes import (
     bfloat16 as bfloat16,
+    itemsize_bits as itemsize_bits,
     canonicalize_dtype as canonicalize_dtype,
-    finfo,  # TODO(phawkins): switch callers to jnp.finfo?
+    finfo,  # TODO(phawkins): switch callers to jnp.finfo?  # noqa: F401
     float0 as float0,
-    iinfo,  # TODO(phawkins): switch callers to jnp.iinfo?
-    issubdtype,  # TODO(phawkins): switch callers to jnp.issubdtype?
+    iinfo,  # TODO(phawkins): switch callers to jnp.iinfo?  # noqa: F401
+    issubdtype,  # TODO(phawkins): switch callers to jnp.issubdtype?  # noqa: F401
     extended as extended,
     prng_key as prng_key,
     result_type as result_type,
