@@ -14,10 +14,11 @@
 """Colocated Python API."""
 
 # Note: import <name> as <name> is required for names to be exported.
-# See PEP 484 & https://github.com/google/jax/issues/7570
+# See PEP 484 & https://github.com/jax-ml/jax/issues/7570
 
 # pylint: disable=useless-import-alias
 from jax.experimental.colocated_python.api import (
     colocated_cpu_devices as colocated_cpu_devices,
     colocated_python as colocated_python,
+    colocated_python_class as colocated_python_class,
 )
