@@ -42,7 +42,27 @@ def colocated_cpu_devices(
 
 
 def colocated_cpu_devices(devices_or_mesh):
-  """Finds devices or a mesh that has CPU devices colocated with the given devices or mesh."""
+  """Finds devices or a mesh that has CPU devices colocated with the given devices or mesh.
+
+  An accelerator device often accompanies a CPU device that is on the same host.
+  Furthermore, when a single host has multiple accelerator devices, there can be
+  multiple CPU devices, each of which is associated with one of the accelerator
+  devices with a 1:1 correspondence.
+
+  This function finds the colocated CPU devices for the given devices or mesh.
+  When the input is a mesh, the returned value is another mesh that has the same
+  shape as the input mesh but has colocated CPU devices. If an input device is
+  already a CPU device, it is returned as-is.
+
+  It preserves ordering. The output CPU device at index i is associated with the
+  input accelerator device at index i.
+
+  Args:
+    devices_or_mesh: A tuple of devices or a mesh.
+
+  Returns:
+    A tuple of devices or a mesh that has the colocated CPU devices.
+  """
   if isinstance(devices_or_mesh, jax.sharding.Mesh):
     return _colocated_cpu_mesh_cached(devices_or_mesh)
 
@@ -85,10 +105,17 @@ def _colocated_cpu_devices_cached(
 def _colocated_cpu_devices_cached_fallback_to_cpu_backend(
     devices: tuple[jax.Device, ...],
 ) -> Sequence[jax.Device]:
-  # PjRt-IFRT currently defines CPU devices by using a CPU backend.
   # TODO(hyeontaek): Remove this fallback path once a PjRt-IFRT backend defines
   # CPU devices by its own instead of using a separate CPU backend.
-  cpu_backend_devices = jax.local_devices(backend="cpu")
+  if devices[0].device_kind == "cpu":
+    # Use the devices from the backend of an original device if it defines CPU
+    # devices.
+    cpu_backend_devices = [d for d in devices[0].client._get_all_devices()
+                           if d.device_kind == "cpu"]
+  else:
+    # PjRt-IFRT on a non-CPU platform currently defines CPU devices on a separae
+    # CPU backend.
+    cpu_backend_devices = jax.devices(backend="cpu")
   device_index_map = {device.id: i for i, device in enumerate(jax.devices())}
 
   available_devices = devices[: min(len(cpu_backend_devices), len(devices))]
@@ -112,12 +139,43 @@ def _colocated_cpu_mesh_cached(mesh: jax.sharding.Mesh) -> jax.sharding.Mesh:
 
 
 def colocated_python(fun: Callable[..., Any]):
-  """Executes the given Python function on the same devices as the arguments."""
+  """Executes the given Python function on the same devices as the arguments.
+
+  The returned colocated Python callable lets the user run a serializable Python
+  function on the same devices as the arguments, potentially on remote hosts.
+
+  Python callable implements `specialize` and `__call__` methods. See their
+  docstrings for details and https://docs.jax.dev/en/latest/notebooks/colocated-python.html
+  for examples.
+
+  Args:
+    fun: An original function to wrap as an I/O callable.
+
+  Returns:
+    Colocated Python callable with no initial specialization.
+  """
   return make_callable(
       fun, api_util.fun_sourceinfo(fun), api_util.fun_signature(fun)
   )
 
 
 def colocated_python_class(cls: type[object]) -> type[object]:
-  """Executes the given Python class methods on the same devices as the arguments."""
+  """Creates a wrapper class that executes the given Python class methods on the same devices as the arguments.
+
+  The wrapper class exposes the returned type's methods, and can be instantiated
+  on JAX. An actual object will be instantiated on the host of the devices of
+  the arguments' when a method of the wrapper instance is called for the first
+  time.
+
+  The actual object will persist while the wrapper object is alive, and will be
+  destroyed asynchronously when the wrapper object is destroyed. Note that if
+  the wrapper object is destroyed immediately without any method call, actual
+  objects will not be created.
+
+  Args:
+    cls: The class to wrap as a colocated Python object.
+
+  Returns:
+    Wrapper class.
+  """
   return wrap_class(cls, api_util.fun_sourceinfo(cls))
