@@ -15,12 +15,14 @@
 
 from __future__ import annotations
 
+from collections.abc import Callable, Sequence
 import dataclasses
 import inspect
 import random
 import threading
 from typing import Any
-from collections.abc import Callable, Sequence
+import uuid
+import weakref
 
 import jax
 from jax._src import api
@@ -31,7 +33,8 @@ from jax._src.lib import xla_client as xc
 from jax._src.traceback_util import api_boundary
 from jax._src.util import wraps
 from jax.experimental.colocated_python import func_backend
-from jax.experimental.colocated_python.serialization import _deserialize_specs, _make_specs_for_serialized_specs, _serialize, _serialize_specs
+from jax.experimental.colocated_python.serialization import _deserialize, _deserialize_specs, _make_specs_for_serialized_specs, _serialize, _serialize_specs
+from jax.extend.backend import register_backend_cache as jax_register_backend_cache
 from jax.extend.ifrt_programs import ifrt_programs
 
 ShapeDtypeStructTree = Any  # PyTree[api.ShapeDtypeStruct]
@@ -186,12 +189,14 @@ def _compile_to_executable(
     # TODO(hyeontaek): Implement colocated Python support in McJAX and remove
     # this fallback path.
     if "PjRtCompiler requires an HloProgram" in str(e):
-      return fun
+      return _deserialize(pickled_function)[0]
     raise
 
 
 def _make_output_specs_and_push_result_fun(
-    info: FunctionInfo, specialization: Specialization, uid: int
+    info: FunctionInfo,
+    specialization: Specialization,
+    uid: int,
 ) -> Callable[..., Any]:
   """Creates a function that computes output specs and pushes the result to the result store."""
   assert specialization.in_specs_treedef is not None
@@ -226,7 +231,9 @@ def _make_output_specs_and_push_result_fun(
 
 
 def _make_pop_result_fun(
-    info: FunctionInfo, specialization: Specialization, uid: int
+    info: FunctionInfo,
+    specialization: Specialization,
+    uid: int,
 ) -> Callable[..., Any]:
   """Makes a function that pops results from the result store."""
   assert specialization.out_specs_treedef is not None
@@ -259,7 +266,8 @@ def _make_pop_result_fun(
 
 
 def _make_async_execution_fun(
-    info: FunctionInfo, specialization: Specialization
+    info: FunctionInfo,
+    specialization: Specialization,
 ) -> Callable[..., Any]:
   """Makes a function that asynchronously executes the function."""
   assert specialization.in_specs_treedef is not None
@@ -280,9 +288,9 @@ def _make_async_execution_fun(
   )
 
 
-@jax._src.util.cache(max_size=None)
-def _get_specialized_func(
-    info: FunctionInfo, specialization: Specialization
+def _uncached_get_specialized_func(
+    info: FunctionInfo,
+    specialization: Specialization,
 ) -> Callable[..., Any]:
   """Returns a specialized function for the given specialization."""
   util.test_event("colocated_python_func._get_specialized_func")
@@ -302,9 +310,14 @@ def _get_specialized_func(
       if async_execution_func is None:
         if specialization.out_specs_treedef is None:
           if specialization.out_specs_fn is None:
-            serialized_out_specs = _make_output_specs_and_push_result_fun(
-                info, specialization, uid
-            )(*args, **kwargs)
+            output_specs_and_push_result_fun = (
+                _make_output_specs_and_push_result_fun(
+                    info, specialization, uid
+                )
+            )
+            serialized_out_specs = output_specs_and_push_result_fun(
+                *args, **kwargs
+            )
 
             # Waits for the output_specs. This may block.
             out_specs_treedef, out_specs_leaves = _deserialize_specs(
@@ -321,6 +334,13 @@ def _get_specialized_func(
                 info, specialization
             )
 
+            # Hold the PyExecutable until async_execution_fun is called at
+            # least once, so the number of _OBJECT_STORE references at the
+            # backend does not drop to 0.
+            async_execution_func.output_specs_and_push_result_fun = (
+                output_specs_and_push_result_fun
+            )
+
             return _make_pop_result_fun(info, specialization, uid)()
           else:
             # Compute out_specs using out_specs_fn and inputs.
@@ -348,119 +368,345 @@ def _get_specialized_func(
 
     # Asynchronous execution runs outside of the mutex to allow concurrent
     # execution for inline executors.
-    return async_execution_func(*args, **kwargs)
+    result = async_execution_func(*args, **kwargs)
+    with mutex:
+      async_execution_func.output_specs_and_push_result_fun = None
+    return result
 
   return specialized_func
 
 
+class _SpecializedCollection:
+  """Collection of specialized functions for a single unspecialized function.
+
+  The `get()` method retrieves the specialized function for the provided input
+  spec, either by looking up a cache or by compiling the specialized function.
+
+  Looking up a cache with an input spec as a key can be slow, because
+  `Sharding`'s equivalence comparison is slow. Instead, we maintain two caches
+  for the same value: we use the ID of the sharding object (via `WeakSpec`) as
+  the key in one cache, and the corresponding strong references to the sharding
+  object (via `StrongSpec`) as the key in another cache. Looking up the
+  `WeakSpec`-keyed cache is fast. Note that the ID integer in the `WeakSpec`
+  cache will remain valid as long as a strong-ref exists in the `StrongSpec`
+  cache.
+
+  The `StrongSpec`-keyed cache is unbounded, while the `WeakSpec`-keyed cache
+  is LRU(1): if there is a miss in the `WeakSpec` cache but a hit in the
+  `StrongSpec` cache, the strong-ref is the `StrongSpec` cache and the ID
+  integer in the `WeakSpec` cache are both updated.
+  """
+
+  @dataclasses.dataclass(slots=True, unsafe_hash=True)
+  class WeakSpec:
+    """WeakSpec stores just the `id()` of the input spec sharding."""
+
+    dtypes: tuple[jax.numpy.dtype, ...]
+    shapes: tuple[tuple[int, ...], ...]
+    sharding_ids: tuple[int, ...]
+    treedef: tree_util.PyTreeDef
+
+    def __init__(
+        self, args_leaves: Sequence[jax.Array], treedef: tree_util.PyTreeDef
+    ):
+      self.dtypes = tuple(x.dtype for x in args_leaves)
+      self.shapes = tuple(x.shape for x in args_leaves)
+      self.sharding_ids = tuple(id(x.sharding) for x in args_leaves)
+      self.treedef = treedef
+
+  @dataclasses.dataclass(slots=True, unsafe_hash=True)
+  class StrongSpec:
+    """StrongSpec stores the full input spec sharding."""
+
+    in_specs_treedef: tree_util.PyTreeDef | None = None
+    in_specs_leaves: tuple[api.ShapeDtypeStruct, ...] | None = None
+
+    def __init__(
+        self, args_leaves: Sequence[jax.Array], pytreedef: tree_util.PyTreeDef
+    ):
+      self.in_specs_leaves = tuple(_get_spec(x) for x in args_leaves)
+      self.in_specs_treedef = pytreedef
+
+  def __init__(self):
+    CompiledId = int
+
+    self._weak_to_id: dict[_SpecializedCollection.WeakSpec, CompiledId] = {}
+    self._id_to_weak: dict[CompiledId, _SpecializedCollection.WeakSpec] = {}
+    self._strong_to_id: dict[_SpecializedCollection.StrongSpec, CompiledId] = {}
+    self._id_to_compiled: dict[CompiledId, Callable[..., Any]] = {}
+
+    self._counter = 0
+    self._mu = threading.Lock()
+
+  def get(
+      self,
+      args_leaves: Sequence[jax.Array],
+      pytreedef: tree_util.PyTreeDef,
+      func_info: FunctionInfo,
+      specialization: Specialization,
+  ) -> Callable[..., Any]:
+    # TODO(hyeontaek): Allow Python values in args_leaves, similar to the todo
+    # in _get_spec().
+
+    # Attempt fast-path cache hit.
+    weak_spec = _SpecializedCollection.WeakSpec(args_leaves, pytreedef)
+    compiled_id = self._weak_to_id.get(weak_spec)
+    if compiled_id is not None:
+      return self._id_to_compiled[compiled_id]
+
+    with self._mu:
+      # Attempt slow-path cache hit.
+      strong_spec = _SpecializedCollection.StrongSpec(args_leaves, pytreedef)
+      compiled_id = self._strong_to_id.pop(strong_spec, None)
+      if compiled_id is not None:
+        # Update the caches so that the fast-path cache stores the `id()` of the
+        # shardings presented by the current invocation.
+        old_weak = self._id_to_weak.pop(compiled_id)
+        del self._weak_to_id[old_weak]
+
+        self._strong_to_id[strong_spec] = compiled_id
+        self._weak_to_id[weak_spec] = compiled_id
+        self._id_to_weak[compiled_id] = weak_spec
+
+        return self._id_to_compiled[compiled_id]
+
+      # Cache-miss: compile.
+      if specialization.devices is None:
+        result = _uncached_get_specialized_func(
+            func_info,
+            specialization.update(
+                in_specs_treedef=strong_spec.in_specs_treedef,
+                in_specs_leaves=strong_spec.in_specs_leaves,
+                devices=_infer_devices_from_args(args_leaves),
+            ),
+        )
+      else:
+        result = _uncached_get_specialized_func(
+            func_info,
+            specialization.update(
+                in_specs_treedef=strong_spec.in_specs_treedef,
+                in_specs_leaves=strong_spec.in_specs_leaves,
+            ),
+        )
+
+      compiled_id = self._counter
+      self._counter += 1
+
+      self._weak_to_id[weak_spec] = compiled_id
+      self._strong_to_id[strong_spec] = compiled_id
+      self._id_to_weak[compiled_id] = weak_spec
+      self._id_to_compiled[compiled_id] = result
+      return result
+
+
+class _JaxSecondLevelCaches:
+  """Manages second-level caches registered as a single cache with JAX."""
+
+  def __init__(self, name: str):
+    self._lock = threading.Lock()
+    self._callbacks: dict[int, Callable[..., Any]] = {}
+    jax_register_backend_cache(self, name)
+
+  def cache_clear(self):
+    """Meant to be invoked by JAX internals."""
+    for callback in self._callbacks.values():
+      callback()
+    self._callbacks.clear()
+
+  def register_second_level(
+      self, uid: int, cache_clear_callback: Callable[..., Any]
+  ):
+    self._callbacks[uid] = cache_clear_callback
+
+  def remove_second_level(self, uid: int):
+    try:
+      self._callbacks.pop(uid)
+    except KeyError:
+      pass
+
+
+class _CachedColocatedFunctionMaker:
+  """Function maker for colocated Python functions.
+
+  Generated functions are stored (cached) indefinitely so that they can be
+  reused, until the cache is dropped.
+  """
+
+  JAX_CACHE = _JaxSecondLevelCaches("colocated_python_specialized_func_cache")
+
+  def __init__(self, held_by: int | None):
+    self.held_by = held_by if held_by is not None else uuid.uuid4().int
+    specialized_collections: list[_SpecializedCollection] = []
+    specialized_functions: list[Callable[..., Any]] = []
+
+    def clear_caches():
+      specialized_collections.clear()
+      specialized_functions.clear()
+
+    _CachedColocatedFunctionMaker.JAX_CACHE.register_second_level(
+        self.held_by,
+        clear_caches,
+    )
+    self.specialized_collections = specialized_collections
+    self.specialized_functions = specialized_functions
+
+  def __del__(self):
+    self.specialized_collections.clear()
+    self.specialized_functions.clear()
+    try:
+      _CachedColocatedFunctionMaker.JAX_CACHE.remove_second_level(self.held_by)
+    except AttributeError:
+      # Ignore error during python finalization.
+      pass
+
+  def _make_callable(
+      self,
+      info: FunctionInfo,
+      specialization: Specialization,
+  ):
+    """Internal implementation of make_callable."""
+
+    def specialize(
+        in_specs: ShapeDtypeStructTree | None = None,
+        out_specs_fn: Callable[..., ShapeDtypeStructTree] | None = None,
+        devices: Sequence[jax.Device] | None = None,
+    ):
+      """Returns a colocated Python callable with extra specialization.
+
+      Args:
+        in_specs: Optionally specifies the expected input specs. Input specs are
+          expressed as a `PyTree[ShapeDtypeStruct]` for `(args, kwargs)` of a
+          function call.
+        out_specs_fn: Optionally specifies a function that computes the output
+          specs from input specs. If unspecified, colocated Python will compute
+          the output specs during the very first execution, and this execution
+          will be synchronous.
+        devices: Optionally specifies the devices to execute the function on.
+          Must be provided if `in_specs` has no leaves because devices cannot be
+          inferred from input specs or arguments.
+
+      Returns:
+        A colocated Python callable with extra specialization.
+      """
+      # TODO(hyeontaek): Allow unspecified devices for zero-leaf `in_specs` if
+      # `out_specs_fn(in_specs)` returns at least one leaf that we can use for
+      # inferring `devices`.
+      if in_specs is None:
+        in_specs_leaves, in_specs_treedef = None, None
+      else:
+        in_specs_leaves_list, in_specs_treedef = tree_util.tree_flatten(
+            in_specs
+        )
+        in_specs_leaves = tuple(in_specs_leaves_list)
+      return self._make_callable(
+          info,
+          specialization.update(
+              in_specs_treedef=in_specs_treedef,
+              in_specs_leaves=in_specs_leaves,
+              out_specs_fn=out_specs_fn,
+              devices=devices,
+          ),
+      )
+
+    # Caches for a collection of specialized functions or a specialized function
+    # itself. The latter is used as a performance optimization when the input
+    # spec is explicitly specified and can skip a collection lookup. The caches
+    # use weakrefs so that we avoid creating cyclic references.
+    specialized_collections_wref = lambda: None
+    specialized_functions_wref = lambda: None
+    wref_mu = threading.Lock()
+
+    @api_boundary
+    def __call__(*args, **kwargs):
+      """Executes the given Python function on the same devices as the arguments or as specialized.
+
+      If the callable has not been specialized with output shapes and shardings
+      (see `specialize` above), the very first call will run synchronously to
+      discover output shapes and shardings, and will run asynchronously after.
+      If specialized with output shapes and shardings, every execution of the
+      callable will be asynchronous.
+      """
+      args_leaves, in_specs_treedef = tree_util.tree_flatten((args, kwargs))
+
+      no_input = len(args_leaves) == 0
+      if no_input and specialization.devices is None:
+        raise ValueError(
+            "No devices found. colocated_python function without input"
+            " arguments must be first specialized with devices."
+        )
+
+      fully_specified_in_spec = (
+          specialization.in_specs_treedef is not None
+          and specialization.in_specs_leaves is not None
+      )
+
+      if not fully_specified_in_spec and not no_input:
+        # We need to handle input polymorphism
+        nonlocal specialized_collections_wref
+        with wref_mu:
+          collection: _SpecializedCollection = specialized_collections_wref()
+          if collection is None:
+            collection = _SpecializedCollection()
+            self.specialized_collections.append(collection)
+            specialized_collections_wref = weakref.ref(collection)
+        result = collection.get(
+            args_leaves, in_specs_treedef, info, specialization
+        )(*args, **kwargs)
+        del collection
+        return result
+
+      # No input polymorphism -- exactly one compiled function is possible.
+      with wref_mu:
+        nonlocal specialized_functions_wref
+        func: Callable[..., Any] = specialized_functions_wref()
+        if func is None:
+          if fully_specified_in_spec and specialization.devices is not None:
+            func = _uncached_get_specialized_func(info, specialization)
+          elif fully_specified_in_spec:
+            func = _uncached_get_specialized_func(
+                info,
+                specialization.update(
+                    devices=_infer_devices_from_args(args_leaves)
+                ),
+            )
+          elif no_input:
+            func = _uncached_get_specialized_func(
+                info,
+                specialization.update(
+                    in_specs_leaves=tuple(),
+                    in_specs_treedef=in_specs_treedef,
+                ),
+            )
+          self.specialized_functions.append(func)
+          specialized_functions_wref = weakref.ref(func)
+      result = func(*args, **kwargs)
+      del func
+      return result
+
+    __call__ = wraps(info.fun)(__call__)
+    __call__.specialize = specialize
+    return __call__
+
+  def make_callable(
+      self,
+      fun: Callable[..., Any],
+      fun_sourceinfo: str | None,
+      fun_signature: inspect.Signature | None,
+  ):
+    """Makes a colocated Python callable."""
+    return self._make_callable(
+        FunctionInfo(fun, fun_sourceinfo, fun_signature), Specialization()
+    )
+
+
+_DEFAULT_FUNCTION_MAKER = _CachedColocatedFunctionMaker(None)
+
+
 def make_callable(
     fun: Callable[..., Any],
     fun_sourceinfo: str | None,
     fun_signature: inspect.Signature | None,
 ):
-  """Makes a colocated Python callable."""
-  return _make_callable(
-      FunctionInfo(fun, fun_sourceinfo, fun_signature), Specialization()
+  return _DEFAULT_FUNCTION_MAKER.make_callable(
+      fun, fun_sourceinfo, fun_signature
   )
-
-
-def _make_callable(info: FunctionInfo, specialization: Specialization):
-  """Internal implementation of make_callable."""
-
-  def specialize(
-      in_specs: ShapeDtypeStructTree | None = None,
-      out_specs_fn: Callable[..., ShapeDtypeStructTree] | None = None,
-      devices: Sequence[jax.Device] | None = None,
-  ):
-    """Returns a colocated Python callable with extra specialization.
-
-    Args:
-      in_specs: Optionally specifies the expected input specs. Input specs are
-        expressed as a `PyTree[ShapeDtypeStruct]` for `(args, kwargs)` of a
-        function call.
-      out_specs_fn: Optionally specifies a function that computes the output
-        specs from input specs. If unspecified, colocated_python will compute
-        the output specs during the very first execution, and this execution
-        will be synchronous.
-      devices: Optionally specifies the devices to execute the function on. Must
-        be provided if in_specs has no leaves because devices cannot be inferred
-        from input specs or arguments.
-
-    Returns:
-      A colocated Python callable with extra specialization.
-    """
-    # TODO(hyeontaek): Allow unspecified devices for zero-leaf `in_specs` if
-    # `out_specs_fn(in_specs)` returns at least one leaf that we can use for
-    # inferring `devices`.
-    if in_specs is None:
-      in_specs_leaves, in_specs_treedef = None, None
-    else:
-      in_specs_leaves_list, in_specs_treedef = tree_util.tree_flatten(in_specs)
-      in_specs_leaves = tuple(in_specs_leaves_list)
-    return _make_callable(
-        info,
-        specialization.update(
-            in_specs_treedef=in_specs_treedef,
-            in_specs_leaves=in_specs_leaves,
-            out_specs_fn=out_specs_fn,
-            devices=devices,
-        ),
-    )
-
-  @api_boundary
-  def __call__(*args, **kwargs):
-    """Executes the function.
-
-    If the output specs are not known, the very first execution will be
-    synchronous.
-    """
-    args_leaves, in_specs_treedef = tree_util.tree_flatten((args, kwargs))
-
-    in_specs_leaves = tuple(_get_spec(x) for x in args_leaves)
-    if specialization.in_specs_treedef is None:
-      # Allow input polymorphism by applying input_specs specialization
-      # temporarily for this call.
-      return _make_callable(
-          info,
-          specialization.update(
-              in_specs_treedef=in_specs_treedef,
-              in_specs_leaves=in_specs_leaves,
-          ),
-      )(*args, **kwargs)
-
-    if specialization.devices is None:
-      devices = _infer_devices_from_args(args_leaves)
-      if devices is None:
-        raise ValueError(
-            "No devices found. colocated_python function without input"
-            " arguments must be first specialized with devices."
-        )
-      # Allow device polymorphism by applying devices specialization temporarily
-      # for this call.
-      return _make_callable(info, specialization.update(devices=devices))(
-          *args, **kwargs
-      )
-
-    # Assertion is added to silence mypy error: Unsupported operand types for !=
-    # ("PyTreeDef" and "None")  [operator]
-    assert isinstance(specialization.in_specs_treedef, tree_util.PyTreeDef)
-
-    # If input_specs is known, verify that it matches actual inputs.
-    if (specialization.in_specs_treedef != in_specs_treedef
-        or specialization.in_specs_leaves != in_specs_leaves):
-      raise ValueError(
-          "Input specs in specialization and input specs of arguments must have"
-          " the same pytree structure, but they have the following structural"
-          " differences:\n"
-          + ("\n".join(
-                f"   - {tree_util.keystr(path)} is a {thing1} in value 1 and"
-                f" a {thing2} in  value 2, so {explanation}.\n"
-                for path, thing1, thing2, explanation in tree_util.equality_errors_pytreedef(
-                    specialization.in_specs_treedef, in_specs_treedef
-                ))))
-
-    return _get_specialized_func(info, specialization)(*args, **kwargs)
-
-  __call__ = wraps(info.fun)(__call__)
-  __call__.specialize = specialize
-  return __call__
