@@ -16,7 +16,7 @@
 from __future__ import annotations
 
 import threading
-from typing import Sequence
+from collections.abc import Sequence
 
 import jax
 
