@@ -44,6 +44,7 @@ from jax._src.lib import _jax
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import func as func_dialect
 from jax._src.lib.mlir.dialects import hlo
+from jax.experimental import roofline
 from jax.experimental.jax2tf import jax2tf as jax2tf_internal
 from jax._src.interpreters import mlir
 import ml_dtypes
@@ -134,7 +135,7 @@ def call_tf(
     args_flat_jax = tuple(map(canonical_arg, args_flat_jax))
     def make_tensorspec(a_jax):
       a_tf_dtype = jax2tf_internal._to_tf_dtype(a_jax.dtype)
-      a_tf_shape = [d if core.is_constant_dim(d) else None for d in a_jax.shape]
+      a_tf_shape = [d if core.is_constant_dim(d) else None for d in getattr(a_jax, "shape", ())]
       return tf.TensorSpec(a_tf_shape, a_tf_dtype)
     args_flat_sig_tf = tuple(map(make_tensorspec, args_flat_jax))
 
@@ -303,10 +304,7 @@ def check_tf_result(idx: int, r_tf: TfVal, r_aval: core.ShapedArray | None) -> T
   # that tf.ensure_shape did this, but it can only take shapes that contain None
   # not computed shapes. However, in eager mode we should be able to resolve
   # the declared shapes to constants and we get better checking.
-  if tf.executing_eagerly():
-    r_aval_shape_tf = jax2tf_internal._eval_shape(r_aval.shape)
-  else:
-    r_aval_shape_tf = jax2tf_internal._aval_to_tf_shape(r_aval)
+  r_aval_shape_tf = jax2tf_internal._aval_to_tf_shape(r_aval)
   # We do as much checking as we can here, instead of relying on tf.ensure_shape
   # because the latter gives different errors in eager vs. compiled mode.
   # TODO(b/279454591): This strange error is from TF. Eager function suppose
@@ -366,8 +364,7 @@ def _call_tf_impl(*args_jax_flat, callable_flat_tf, **_):
       res_tf_platform = tf.DeviceSpec.from_string(res_tf.backing_device).device_type
       res_jax_platform = res_tf_platform.lower()
       if res_jax_platform in _DLPACK_PLATFORMS:
-        res_dlpack = tf.experimental.dlpack.to_dlpack(res_tf)
-        return jax.dlpack.from_dlpack(res_dlpack)
+        return jax.dlpack.from_dlpack(res_tf)
 
     # When working with a bfloat16 scalar tf.Tensor,np.asarray() can fail.
     # To handle this special case, we create a numpy copy.
@@ -471,7 +468,7 @@ call_tf_p.def_effectful_abstract_eval(_call_tf_abstract_eval)
 def _mlir_type_to_numpy_dtype(type: ir.Type) -> np.dtype:
   """Converts an MLIR scalar type to a NumPy dtype."""
 
-  if ir.IntegerType.isinstance(type):
+  if isinstance(type, ir.IntegerType):
     type = ir.IntegerType(type)
     width = type.width
     if width == 1:
@@ -487,20 +484,20 @@ def _mlir_type_to_numpy_dtype(type: ir.Type) -> np.dtype:
     else:
       raise ValueError(f"Unsupported integer width: {width}")
 
-  elif ir.F16Type.isinstance(type):
+  elif isinstance(type, ir.F16Type):
     return np.dtype(np.float16)
-  elif ir.F32Type.isinstance(type):
+  elif isinstance(type, ir.F32Type):
     return np.dtype(np.float32)
-  elif ir.F64Type.isinstance(type):
+  elif isinstance(type, ir.F64Type):
     return np.dtype(np.float64)
-  elif ir.BF16Type.isinstance(type):
+  elif isinstance(type, ir.BF16Type):
     return np.dtype(ml_dtypes.bfloat16)
 
-  elif ir.ComplexType.isinstance(type):
+  elif isinstance(type, ir.ComplexType):
     element_type = ir.ComplexType(type).element_type
-    if ir.F32Type.isinstance(element_type):
+    if isinstance(element_type, ir.F32Type):
       return np.dtype(np.complex64)
-    elif ir.F64Type.isinstance(element_type):
+    elif isinstance(element_type, ir.F64Type):
       return np.dtype(np.complex128)
     else:
       raise ValueError(f"Unsupported complex element type: {element_type}")
@@ -647,16 +644,6 @@ def _register_call_lowering(platform):
 for platform in ("cpu", "cuda", "tpu"):
   _register_call_lowering(platform)
 
-# Support the call_tf under jax2tf.convert in eager mode
-def _jax2tf_call_tf(*args: TfVal,
-                    callable_flat_tf: Callable,
-                    **_) -> TfVal:
-  with jax2tf_internal.inside_call_tf():
-    res_tf_flat = callable_flat_tf(*args)
-  return res_tf_flat
-
-jax2tf_internal.tf_impl[call_tf_p] = _jax2tf_call_tf
-
 
 def emit_tf_embedded_graph_custom_call(
     ctx: mlir.LoweringRuleContext,
@@ -727,3 +714,8 @@ def add_to_call_tf_concrete_function_list(concrete_tf_fn: Any, call_tf_concrete_
     called_index = len(call_tf_concrete_function_list)
     call_tf_concrete_function_list.append(concrete_tf_fn)
   return called_index
+
+# Register a roofline call so that users can use roofline on functions that
+# contain call_tf. We register roofline in this file (instead of within the
+# roofline module) to avoid having to import jax2tf in roofline.
+roofline.register_standard_roofline(call_tf_p)
