@@ -381,6 +381,9 @@ class NativeSerializationImpl:
     return _export._get_vjp_fun(self.fun_jax,
                                 in_tree=self.exported.in_tree,
                                 in_avals=self.exported.in_avals,
+                                has_named_shardings=self.exported._has_named_shardings,
+                                in_named_shardings=self.exported._in_named_shardings,
+                                out_named_shardings=self.exported._out_named_shardings,
                                 in_shardings_hlo=self.exported.in_shardings_hlo,
                                 out_avals=self.exported.out_avals,
                                 out_shardings_hlo=self.exported.out_shardings_hlo,
