@@ -15,8 +15,9 @@
 from __future__ import annotations
 
 from collections import defaultdict
-from functools import partial, reduce, total_ordering, wraps
-from typing import Any, Callable, Iterator, NamedTuple
+from collections.abc import Callable, Iterator
+from functools import partial, reduce, total_ordering
+from typing import Any, NamedTuple
 
 import jax
 from jax import lax
@@ -24,7 +25,6 @@ from jax import tree_util
 from jax.errors import KeyReuseError
 from jax.interpreters import batching, mlir
 from jax._src import api_util
-from jax._src import config
 from jax._src import core
 from jax._src import linear_util as lu
 from jax._src import pjit
@@ -35,10 +35,12 @@ from jax._src import traceback_util
 from jax._src import util
 from jax._src.ad_checkpoint import remat_p
 from jax._src.debugging import debug_callback_p
+from jax._src.effects import Effect
+from jax._src.hashable_array import HashableArray
 from jax._src.interpreters import partial_eval as pe
 from jax._src.util import weakref_lru_cache
 
-from jax.experimental.shard_map import shard_map_p
+from jax._src.shard_map import shard_map_p
 import numpy as np
 
 
@@ -212,13 +214,13 @@ def key_reuse_signature_from_eqn(eqn: core.JaxprEqn) -> KeyReuseSignature:
       return sig.signature(eqn)
     else:
       raise TypeError(
-        f"Unrecognized key reuse sigature of type {type(sig)}: {sig}")
+        f"Unrecognized key reuse signature of type {type(sig)}: {sig}")
   else:
     return unknown_signature(eqn)
 
 
 def key_reuse_signature_from_primitive(prim, *args, **params):
-  if prim == pjit.pjit_p:
+  if prim == pjit.jit_p:
     return jaxpr_type_signature(params['jaxpr'].jaxpr)
   if prim not in key_reuse_signatures:
     # TODO(jakevdp) should we generate an unknown signature here?
@@ -231,12 +233,13 @@ def key_reuse_signature_from_primitive(prim, *args, **params):
     return jaxpr_type_signature(jaxpr)
   else:
     raise TypeError(
-      f"Unrecognized key reuse sigature of type {type(sig)}: {sig}")
+      f"Unrecognized key reuse signature of type {type(sig)}: {sig}")
 
 
+consume_effect = Effect()
 consume_p = core.Primitive("consume")
 consume_p.def_impl(lambda x: x)
-consume_p.def_abstract_eval(lambda x: x)
+consume_p.def_effectful_abstract_eval(lambda x: (x, {consume_effect}))
 batching.defvectorized(consume_p)
 mlir.register_lowering(
     consume_p,
@@ -246,10 +249,11 @@ def consume(key):
   """Consume the key and return a consumed copy."""
   return consume_p.bind(key)
 
+assert_effect = Effect()
 
 assert_consumed_value_p = core.Primitive("assert_consumed_value")
 assert_consumed_value_p.def_impl(lambda x, *, value: x)
-assert_consumed_value_p.def_abstract_eval(lambda x, *, value: x)
+assert_consumed_value_p.def_effectful_abstract_eval(lambda x, *, value: (x, {assert_effect}))
 batching.defvectorized(assert_consumed_value_p)
 mlir.register_lowering(
     assert_consumed_value_p,
@@ -257,16 +261,16 @@ mlir.register_lowering(
 
 def assert_unconsumed(key):
   """Assert that a key is unconsumed"""
-  assert_consumed_value_p.bind(key, value=False)
+  assert_consumed_value_p.bind(key, value=HashableArray(False))
 
 def assert_consumed(key, value=True):
   """Assert that a key is consumed"""
-  assert_consumed_value_p.bind(key, value=value)
+  assert_consumed_value_p.bind(key, value=HashableArray(value))
 
 
 def _check_consumed_value(eqn, consumed):
   """Extra check for use with assert_consumed_value_p"""
-  expected =  eqn.params['value']
+  expected = eqn.params['value'].val
   if not np.all(consumed == expected):
     if np.all(expected):
       raise AssertionError(f"Expected key to be consumed in {eqn}")
@@ -292,7 +296,6 @@ key_reuse_signatures[random.random_gamma_p] = KeyReuseSignature(Sink(0))
 key_reuse_signatures[lax.broadcast_in_dim_p] = KeyReuseSignature(Forward(0, 0))
 key_reuse_signatures[lax.copy_p] = KeyReuseSignature(Forward(0, 0))
 key_reuse_signatures[lax.convert_element_type_p] = KeyReuseSignature(Forward(0, 0))
-key_reuse_signatures[lax.device_put_p] = KeyReuseSignature(Forward(0, 0))
 key_reuse_signatures[lax.reshape_p] = KeyReuseSignature(Forward(0, 0))
 key_reuse_signatures[lax.squeeze_p] = KeyReuseSignature(Forward(0, 0))
 key_reuse_signatures[prng.random_wrap_p] = KeyReuseSignature(Source(0))
@@ -398,8 +401,11 @@ def jaxpr_type_signature(jaxpr: core.Jaxpr) -> KeyReuseSignature:
 def function_type_signature(fun: Callable[..., Any], *args: Any) -> KeyReuseSignature:
   args_flat, in_tree = tree_util.tree_flatten(args)
   in_avals_flat = [core.get_aval(arg) for arg in args_flat]
-  wrapped_fun, _ = api_util.flatten_fun_nokwargs(lu.wrap_init(fun), in_tree)
-  jaxpr, _, _, () = pe.trace_to_jaxpr_dynamic(wrapped_fun, in_avals_flat)
+  wrapped_fun, _ = api_util.flatten_fun_nokwargs(
+      lu.wrap_init(fun,
+                   debug_info=api_util.debug_info("key_reuse", fun, args, {})),
+      in_tree)
+  jaxpr, _, _ = pe.trace_to_jaxpr_dynamic(wrapped_fun, in_avals_flat)
   return jaxpr_type_signature(jaxpr)
 
 
@@ -413,7 +419,7 @@ def check_key_reuse(fun: Callable[..., Any], /, *args: Any) -> None:
   function_type_signature(fun, *args)
 
 
-#----------------------------------------------------------------------------------
+# ----------------------------------------------------------------------------------
 # key reuse rules for particular primitives:
 
 @dynamic_key_reuse_signature
@@ -448,7 +454,7 @@ key_reuse_signatures[lax.concatenate_p] = _concatenate_signature
 def _pjit_key_type_signature(eqn):
   return jaxpr_type_signature(eqn.params['jaxpr'].jaxpr)
 
-key_reuse_signatures[pjit.pjit_p] = _pjit_key_type_signature
+key_reuse_signatures[pjit.jit_p] = _pjit_key_type_signature
 
 @dynamic_key_reuse_signature
 def _shard_map_type_signature(eqn):
@@ -561,13 +567,21 @@ def _remat_key_type_signature(eqn):
 key_reuse_signatures[remat_p] = _remat_key_type_signature
 
 
+@dynamic_key_reuse_signature
+def _device_put_signature(eqn):
+  num_vals = len(eqn.invars)
+  return KeyReuseSignature(*(Forward(i, i) for i in range(num_vals)))
+
+key_reuse_signatures[lax.device_put_p] = _device_put_signature
+
+
 def call_impl_with_key_reuse_checks(prim: core.Primitive, raw_impl: Callable[..., Any], *args, **kwargs) -> Any:
   if prim not in key_reuse_signatures:
     # TODO(jakevdp): should we use an unknown signature here?
     return raw_impl(*args, **kwargs)
   signature = key_reuse_signature_from_primitive(prim, *args, **kwargs)
-  funcname = "jit-compiled function" if prim == pjit.pjit_p else str(prim)
-  consts = kwargs['jaxpr'].consts if prim == pjit.pjit_p else []
+  funcname = "jit-compiled function" if prim == pjit.jit_p else str(prim)
+  consts = kwargs['jaxpr'].consts if prim == pjit.jit_p else []
   signature.check_signature(*args, *consts, funcname=funcname)
   result = raw_impl(*args, **kwargs)
   signature.update_consumption([*args, *consts], result if prim.multiple_results else [result])
