@@ -13,6 +13,9 @@
 # limitations under the License.
 
 from jax._src.layout import (
-    DeviceLocalLayout as DeviceLocalLayout,
-    Layout as Layout
+    Layout as Layout,
+    Format as Format,
+)
+from jax._src.pjit import (
+    with_layout_constraint as with_layout_constraint,
 )
