@@ -14,4 +14,5 @@
 # ==============================================================================
 """Contains bindings for Mosaic."""
 
-from jax._src.tpu_custom_call import as_tpu_kernel
+from jax._src.tpu_custom_call import as_tpu_kernel as as_tpu_kernel
+from jax._src.tpu_custom_call import lower_module_to_custom_call as lower_module_to_custom_call
