@@ -27,13 +27,16 @@ from .core import (
     TMEM as TMEM,
     Union as Union,
     as_gpu_kernel as as_gpu_kernel,
+    as_torch_gpu_kernel as as_torch_gpu_kernel,
     supports_cross_device_collectives as supports_cross_device_collectives,
 )
 
 from .launch_context import (
+    AsyncCopyImplementation as AsyncCopyImplementation,
+    GLOBAL_BROADCAST as GLOBAL_BROADCAST,
     LaunchContext as LaunchContext,
     MemRefTransform as MemRefTransform,
-    ReductionOp as ReductionOp,
+    TMAReductionOp as TMAReductionOp,
     Rounding as Rounding,
     TileTransform as TileTransform,
     TransposeTransform as TransposeTransform,
@@ -47,18 +50,10 @@ from .layout_inference import (
     infer_layout as infer_layout,
 )
 
-from .layout_inference2 import (
-    infer_layout as infer_layout2,  # noqa: F401
-)
-
 from .layouts import (
     to_layout_attr as to_layout_attr,
 )
 
-from .transform_inference import (
-    infer_transforms as infer_transforms,
-)
-
 from .fragmented_array import (
     FragmentedArray as FragmentedArray,
     FragmentedLayout as FragmentedLayout,
@@ -68,11 +63,18 @@ from .fragmented_array import (
     TCGEN05_COL_LAYOUT as TCGEN05_COL_LAYOUT,
     TiledLayout as TiledLayout,
     WGMMA_LAYOUT as WGMMA_LAYOUT,
+    WGMMA_LAYOUT_8BIT as WGMMA_LAYOUT_8BIT,
     WGMMA_ROW_LAYOUT as WGMMA_ROW_LAYOUT,
     WGMMA_COL_LAYOUT as WGMMA_COL_LAYOUT,
     WGMMA_TRANSPOSED_LAYOUT as WGMMA_TRANSPOSED_LAYOUT,
+    WGMMA_LAYOUT_UPCAST_2X as WGMMA_LAYOUT_UPCAST_2X,
+    WGMMA_LAYOUT_UPCAST_4X as WGMMA_LAYOUT_UPCAST_4X,
+    TMEM_NATIVE_LAYOUT as TMEM_NATIVE_LAYOUT,
+    TMA_GATHER_INDICES_LAYOUT as TMA_GATHER_INDICES_LAYOUT,
+    tmem_native_layout as tmem_native_layout,
     WGSplatFragLayout as WGSplatFragLayout,
     WGStridedFragLayout as WGStridedFragLayout,
+    copy_tiled as copy_tiled,
     optimization_barrier as optimization_barrier,
 )
 from .utils import (
@@ -84,6 +86,7 @@ from .utils import (
     Partition1D as Partition1D,
     SemaphoreRef as SemaphoreRef,
     ThreadSubset as ThreadSubset,
+    MultimemReductionOp as MultimemReductionOp,
     bitwidth as bitwidth,
     bytewidth as bytewidth,
     c as c,
@@ -98,16 +101,22 @@ from .utils import (
     memref_transpose as memref_transpose,
     memref_unfold as memref_unfold,
     memref_unsqueeze as memref_unsqueeze,
+    nanosleep as nanosleep,
+    query_cluster_cancel as query_cluster_cancel,
     single_thread as single_thread,
     single_thread_predicate as single_thread_predicate,
-    system_memory_barrier as system_memory_barrier,
     thread_idx as thread_idx,
     tile_shape as tile_shape,
+    try_cluster_cancel as try_cluster_cancel,
     warp_idx as warp_idx,
     warpgroup_barrier as warpgroup_barrier,
     warpgroup_idx as warpgroup_idx,
     when as when,
 )
+from .mma import (
+    MMALayouts as MMALayouts,
+    mma as mma,
+)
 from .wgmma import (
     WGMMAAccumulator as WGMMAAccumulator,
     wgmma as wgmma,
