@@ -95,7 +95,7 @@ def out_tmem_layouts(op: MlirOperation) -> Sequence[ir.Attribute]:
 def should_have_in_tmem_layout(op: MlirOperation) -> bool:
   """Returns 'true' if the operation operands should be assigned a TMEM layout."""
   return any(
-      ir.MemRefType.isinstance(v.type) and utils.is_tmem_ref(v)
+      isinstance(v.type, ir.MemRefType) and utils.is_tmem_ref(v)
       for v in op.operands
   )
 
@@ -103,7 +103,7 @@ def should_have_in_tmem_layout(op: MlirOperation) -> bool:
 def should_have_out_tmem_layout(op: MlirOperation) -> bool:
   """Returns 'true' if the operation results should be assigned a TMEM layout."""
   return any(
-      ir.MemRefType.isinstance(v.type) and utils.is_tmem_ref(v)
+      isinstance(v.type, ir.MemRefType) and utils.is_tmem_ref(v)
       for v in op.results
   )
 
@@ -123,12 +123,12 @@ def has_out_tmem_layouts_set(op: MlirOperation) -> bool:
 
 def should_have_in_layout(op: MlirOperation) -> bool:
   """Returns 'true' if the operation operands should be assigned a layout."""
-  return any(ir.VectorType.isinstance(v.type) for v in op.operands)
+  return any(isinstance(v.type, ir.VectorType) for v in op.operands)
 
 
 def should_have_out_layout(op: MlirOperation) -> bool:
   """Returns 'true' if the operation results should be assigned a layout."""
-  return any(ir.VectorType.isinstance(v.type) for v in op.results)
+  return any(isinstance(v.type, ir.VectorType) for v in op.results)
 
 
 def should_have_layout(op: MlirOperation) -> bool:
@@ -178,7 +178,7 @@ def _in_attr_for_operand(
     attr_name: str,
 ) -> ir.Attribute | None:
   if attr_name == "in_layouts":
-    predicate = lambda v: ir.VectorType.isinstance(v.type)
+    predicate = lambda v: isinstance(v.type, ir.VectorType)
   elif attr_name == "in_transforms":
     predicate = is_transformable_smem_memref
   else:
@@ -216,7 +216,7 @@ def is_transformable_smem_memref(v: ir.Value) -> bool:
   """Whether the value is a memref in SMEM on which transforms should be applied."""
   barrier_ty = ir.Type.parse("!mosaic_gpu.barrier")
   return (
-      ir.MemRefType.isinstance(v.type)
+      isinstance(v.type, ir.MemRefType)
       # barriers have no business being transformed
       and v.type.element_type != barrier_ty  # pylint: disable=attribute-error
       and utils.is_smem_ref(v)
@@ -225,7 +225,7 @@ def is_transformable_smem_memref(v: ir.Value) -> bool:
 
 def _value_attr(value: ir.Value, attr_type: str) -> ir.Attribute | None:
   if attr_type == "layouts":
-    predicate = lambda v: ir.VectorType.isinstance(v.type)
+    predicate = lambda v: isinstance(v.type, ir.VectorType)
   elif attr_type == "transforms":
     predicate = is_transformable_smem_memref
   else:
@@ -264,7 +264,7 @@ def value_layout(value: ir.Value) -> ir.Attribute | None:
   Raises:
     ValueError: If `result` is not a Vector.
   """
-  if not ir.VectorType.isinstance(value.type):
+  if not isinstance(value.type, ir.VectorType):
     raise ValueError(f"{value} is not a vector.")
 
   return _value_attr(value, "layouts")
@@ -276,7 +276,7 @@ def value_transforms(value: ir.Value) -> ir.Attribute | None:
   Raises:
     ValueError: If `result` is not a memref.
   """
-  if not ir.MemRefType.isinstance(value.type):
+  if not isinstance(value.type, ir.MemRefType):
     raise ValueError(f"{value} is not a memref.")
 
   return _value_attr(value, "transforms")
