@@ -50,7 +50,7 @@ def attn_forward_kernel(
     # Load q: it will stay in L1 throughout. Indices form a matrix because we
     # read, compute, and write all in 2d chunks. 1 element ~= 1 CUDA thread index.
     # q tile has shape [block_h, head_dim].
-    q = pl.load(q_ref, (q_slice, pl.ds(None)), mask=q_mask)
+    q = plgpu.load(q_ref.at[q_slice, :], mask=q_mask)
 
     def _dot(a, b):
       # if a.shape[0] == 1:
@@ -66,7 +66,7 @@ def attn_forward_kernel(
       o_prev, m_prev, l_prev = carry
       curr_k_slice = pl.ds(start_k * block_k, block_k)
 
-      k = pl.load(k_ref, (curr_k_slice, slice(None)))
+      k = k_ref[curr_k_slice, :]
       qk = _dot(q, k.T)  # [block_h, block_k]
       if sm_scale != 1.0:
         qk *= sm_scale  # [block_h, block_k]
@@ -86,7 +86,7 @@ def attn_forward_kernel(
       )  # Use m_next instead of m_curr to avoid a correction on l_curr
       l_curr = s_curr.sum(axis=-1)
       l_next = l_prev_corr + l_curr
-      v = pl.load(v_ref, (curr_k_slice, slice(None)))
+      v = v_ref[curr_k_slice, :]
       o_curr = _dot(s_curr.astype(v.dtype), v)
 
       # flash2 unscaled_o
@@ -106,10 +106,10 @@ def attn_forward_kernel(
 
   start_idx = split_k_seq_len * prog_j
   if start_idx_ref is not None:
-    start_idx = jnp.maximum(start_idx, pl.load(start_idx_ref, ()))
+    start_idx = jnp.maximum(start_idx, start_idx_ref[()])
   kv_seq_len = (prog_j + 1) * split_k_seq_len  # lower bound on actual k_seq_len
   if kv_seq_len_ref is not None:
-    kv_seq_len = jnp.minimum(kv_seq_len, pl.load(kv_seq_len_ref, ()))
+    kv_seq_len = jnp.minimum(kv_seq_len, kv_seq_len_ref[()])
 
   if start_idx_ref is None and kv_seq_len is None:
     o, m_i, l_i = _compute(start_idx, kv_seq_len, o, m_i, l_i)
@@ -122,10 +122,10 @@ def attn_forward_kernel(
   if residual_refs:
     l_ref, m_ref = residual_refs
     vec_q_mask = q_mask.reshape(-1) if q_mask is not None else None
-    pl.store(l_ref, q_slice, l_i, mask=vec_q_mask)
-    pl.store(m_ref, q_slice, m_i, mask=vec_q_mask)
+    plgpu.store(l_ref.at[q_slice], l_i, mask=vec_q_mask)
+    plgpu.store(m_ref.at[q_slice], m_i, mask=vec_q_mask)
   o = o.astype(o_ref.dtype)
-  pl.store(o_ref, (q_slice, pl.ds(None)), o, mask=q_mask)
+  plgpu.store(o_ref.at[q_slice, :], o, mask=q_mask)
 
 
 def decode_attn_unbatched(
@@ -143,7 +143,8 @@ def decode_attn_unbatched(
     grid: tuple[int, ...] | None,
     interpret: bool,
     debug: bool,
-    return_residuals: bool
+    return_residuals: bool,
+    normalize_output: bool
 ):
   num_heads, head_dim = q.shape
   k_seq_len, _ = k.shape
@@ -192,7 +193,7 @@ def decode_attn_unbatched(
       pl.BlockSpec((None, block_h), lambda i, j: (j, i)),  # l
       pl.BlockSpec((None, block_h), lambda i, j: (j, i)),  # m
     ],
-    compiler_params=plgpu.TritonCompilerParams(
+    compiler_params=plgpu.CompilerParams(
       num_warps=num_warps_, num_stages=num_stages
     ),
     out_shape=[
@@ -218,7 +219,9 @@ def decode_attn_unbatched(
   o = o * correction[:, :, None].astype(o.dtype)
   l_next = (l * correction).sum(axis=0)
   eps = jnp.finfo(l_next.dtype).eps
-  o = o.sum(axis=0) / (l_next[:, None].astype(o.dtype) + eps)
+  o = o.sum(axis=0)
+  if normalize_output:
+    o /= (l_next[:, None].astype(o.dtype) + eps)
   if return_residuals:
     return o, (l_next, m_next)
   else:
@@ -237,7 +240,8 @@ def decode_attn_unbatched(
         "grid",
         "interpret",
         "debug",
-        "return_residuals"
+        "return_residuals",
+        "normalize_output"
     ],
 )
 def mqa(
@@ -255,7 +259,8 @@ def mqa(
     grid: tuple[int, ...] | None = None,
     interpret: bool = False,
     debug: bool = False,
-    return_residuals: bool = False
+    return_residuals: bool = False,
+    normalize_output: bool = True,
 ):
   sm_scale = sm_scale if sm_scale is not None else (1 / math.sqrt(q.shape[-1]))
   bs = q.shape[0]
@@ -274,7 +279,8 @@ def mqa(
       grid=grid,
       interpret=interpret,
       debug=debug,
-      return_residuals=return_residuals
+      return_residuals=return_residuals,
+      normalize_output=normalize_output,
   )
   return jax.vmap(inner)(q, k, v, start_idx, kv_seq_len)
 
@@ -291,7 +297,8 @@ def mqa(
         "grid",
         "interpret",
         "debug",
-        "return_residuals"
+        "return_residuals",
+        "normalize_output"
     ],
 )
 def gqa(
@@ -310,7 +317,12 @@ def gqa(
     interpret: bool = False,
     debug: bool = False,
     return_residuals: bool = False,
+    normalize_output: bool = True,
 ):
+  if not normalize_output and not return_residuals:
+    raise NotImplementedError(
+        "When normalize_output is False, attention residuals must be returned."
+    )
   sm_scale = sm_scale if sm_scale is not None else (1 / math.sqrt(q.shape[-1]))
   batch_size, q_heads, head_dim = q.shape
   k_seq_len, kv_heads = k.shape[1], k.shape[2]
@@ -344,22 +356,28 @@ def gqa(
       interpret=interpret,
       debug=debug,
       return_residuals=return_residuals,
+      normalize_output=normalize_output,
   )
   with_kv_heads = jax.vmap(inner)
-  o, *res = jax.vmap(with_kv_heads)(
+  outputs = jax.vmap(with_kv_heads)(
       q_reshaped, k_transposed, v_transposed, start_idx, kv_seq_len
   )
-  o = o.reshape(batch_size, q_heads, head_dim)
   if return_residuals:
-    l, m = res[0]
+    o, (l, m) = outputs
+    o = o.reshape(batch_size, q_heads, head_dim)
     l = l.reshape(batch_size, q_heads)
     m = m.reshape(batch_size, q_heads)
     return o, (l, m)
   else:
+    o = outputs
+    o = o.reshape(batch_size, q_heads, head_dim)  # pytype: disable=attribute-error
     return o
 
 
-@functools.partial(jax.jit, static_argnames=["sm_scale", "return_residuals"])
+@functools.partial(
+    jax.jit,
+    static_argnames=["sm_scale", "return_residuals", "normalize_output"],
+)
 def mqa_reference(
     q,                # [bs, num_q_heads, head_dim]
     k,                # [bs, k_seq_len, head_dim]
@@ -367,7 +385,8 @@ def mqa_reference(
     start_idx=None,   # [bs]
     kv_seq_len=None,  # [bs]
     sm_scale=None,
-    return_residuals=False
+    return_residuals=False,
+    normalize_output=True,
 ):
   original_dtype = q.dtype
   q = q.astype(jnp.float32)
@@ -389,7 +408,8 @@ def mqa_reference(
   m = logits.max(axis=-1)
   s = jnp.exp(logits - m[..., None])
   l = s.sum(axis=-1)
-  s = s / l[..., None]
+  if normalize_output:
+    s = s / l[..., None]
   o = jnp.einsum("bns,bsd->bnd", s, v).astype(original_dtype)
 
   if return_residuals:
@@ -423,7 +443,10 @@ def mha_reference(
   return jnp.einsum("bns,bsnd->bnd", weights, v)
 
 
-@functools.partial(jax.jit, static_argnames=["sm_scale", "return_residuals"])
+@functools.partial(
+    jax.jit,
+    static_argnames=["sm_scale", "return_residuals", "normalize_output"],
+)
 def gqa_reference(
     q,                # [bs, num_q_heads, head_dim]
     k,                # [bs, k_seq_len, num_k_heads, head_dim]
@@ -431,7 +454,8 @@ def gqa_reference(
     start_idx=None,   # [bs]
     kv_seq_len=None,  # [bs]
     sm_scale=None,
-    return_residuals=False
+    return_residuals=False,
+    normalize_output=True
 ):
   original_dtype = q.dtype
   q = q.astype(jnp.float32)
@@ -466,7 +490,8 @@ def gqa_reference(
   m = logits.max(axis=-1)
   s = jnp.exp(logits - m[..., None])
   l = s.sum(axis=-1)
-  s = s / l[..., None]
+  if normalize_output:
+    s = s / l[..., None]
   o = jnp.einsum("bkgs,bksd->bkgd", s, v_transposed).astype(original_dtype)
   o = o.reshape(bs, num_q_heads, head_dim)
 
