@@ -16,10 +16,11 @@
 
 from __future__ import annotations
 
-from collections.abc import Callable, Mapping
+from collections.abc import Callable
 import dataclasses
 import enum
 import functools
+import json
 from typing import Any, Literal, NamedTuple, Optional, Union, overload
 
 import jax
@@ -91,7 +92,6 @@ MaskFunctionType = Callable[..., jax.Array]
 
 
 def get_kernel_name(
-    block_metadata: Mapping[str, Any],
     is_mqa: bool,
     save_residuals: bool,
     is_segmented: bool,
@@ -101,16 +101,10 @@ def get_kernel_name(
   assert phase == "dq" or phase == "dkv" or phase == "fwd"
   # Saving residuals is supported only for the fwd phase.
   assert not save_residuals or phase == "fwd"
-  residuals = ""
-  if save_residuals:
-    residuals = "_residuals"
-  elif phase == "fwd":
-    residuals = "_no_residuals"
+  residuals = "_residuals" if save_residuals else "_no_residuals"
   attention_type = "mqa" if is_mqa else "mha"
   segments = "_segmented" if is_segmented else ""
-  return f"splash_{attention_type}_{phase}{segments}{residuals}_" + "_".join(
-      f"{k}={v}" for k, v in sorted(block_metadata.items())
-  )
+  return f"splash_{attention_type}_{phase}{segments}{residuals}"
 
 
 # Reference attention implementations
@@ -644,8 +638,8 @@ def _apply_mask_and_soft_cap(
 
       repeats, rem = divmod(k_slice.size, NUM_LANES)
       assert rem == 0
-      q_sequence = pltpu.repeat(
-          q_sequence_ref[...], repeats, axis=1
+      q_sequence = jnp.tile(
+          q_sequence_ref[...], (1, repeats)
       )  # [bq, k_slice.size]
     else:
       assert q_sequence_ref.shape == (NUM_SUBLANES, bq)
@@ -671,14 +665,14 @@ def _apply_mask_and_soft_cap(
       repeats, rem = divmod(kv_ids.shape[1], NUM_LANES)
       if rem:
         raise NotImplementedError(f"block_kv must be a multiple of {NUM_LANES}")
-      q_ids = pltpu.repeat(q_segment_ids_ref[:], repeats, axis=1)  # [bq, bkv]
+      q_ids = jnp.tile(q_segment_ids_ref[:], (1, repeats))  # [bq, bkv]
     else:
       assert bq == q_segment_ids_ref.shape[-1]
       repeats, rem = divmod(bq, NUM_LANES)
       if rem:
         raise NotImplementedError(f"block_q must be a multiple of {NUM_LANES}")
-      kv_ids = pltpu.repeat(
-          kv_segment_ids_ref[k_slice, :], repeats, axis=1
+      kv_ids = jnp.tile(
+          kv_segment_ids_ref[k_slice, :], (1, repeats)
       )  # [k_slice, bq]
       q_ids = q_segment_ids_ref[:1, :]  # [1, bq]
     masks.append(q_ids == kv_ids)
@@ -807,7 +801,7 @@ def flash_attention_kernel(
           f"{bkv_compute=} should be a multiple of {NUM_LANES}"
       )
 
-    s_curr = jnp.exp(qk - pltpu.repeat(m_next, bkv_repeats, axis=1))
+    s_curr = jnp.exp(qk - jnp.tile(m_next, (1, bkv_repeats)))
     assert s_curr.shape == (bq, bkv_compute)
 
     l_curr = jax.lax.broadcast_in_dim(s_curr.sum(axis=-1), l_prev.shape, (0,))
@@ -825,8 +819,8 @@ def flash_attention_kernel(
     v = v.astype(float32)
     o_curr = lax.dot_general(s_curr, v, sv_dims)
 
-    alpha_o = pltpu.repeat(
-        alpha, head_dim_v_repeats, axis=1)[..., :o_scratch_ref.shape[-1]]
+    alpha_o = jnp.tile(
+        alpha, (1, head_dim_v_repeats))[..., :o_scratch_ref.shape[-1]]
     o_scratch_ref[:] = alpha_o * o_scratch_ref[:] + o_curr
 
   @pl.when(should_run)
@@ -840,8 +834,8 @@ def flash_attention_kernel(
   @pl.when(j == grid_width - 1)
   def end():
     l = l_scratch_ref[...]
-    l_inv = pltpu.repeat(
-        1.0 / l, head_dim_v_repeats, axis=1)[..., :o_scratch_ref.shape[-1]]
+    l_inv = jnp.tile(
+        1.0 / l, (1, head_dim_v_repeats))[..., :o_scratch_ref.shape[-1]]
     o_ref[...] = (o_scratch_ref[...] * l_inv).astype(o_ref.dtype)
     if logsumexp_ref is not None:
       assert logsumexp_ref.shape == (bq, NUM_LANES)
@@ -1126,12 +1120,12 @@ def _splash_attention_forward(
     out_specs += [None]
 
   kernel_name = get_kernel_name(
-      dataclasses.asdict(block_sizes),
       is_mqa=is_mqa,
       save_residuals=save_residuals,
       is_segmented=segment_ids is not None,
       phase="fwd",
   )
+  metadata = {"xprof_metadata": json.dumps(dataclasses.asdict(block_sizes))}
 
   if fwd_mask_info.data_next is not None:
     grid_width = fwd_mask_info.data_next.shape[-1]
@@ -1167,6 +1161,7 @@ def _splash_attention_forward(
         out_shape=out_shapes,
         name=kernel_name,
         interpret=interpret,
+        metadata=metadata,
     )(
         fwd_mask_info.data_next,
         fwd_mask_info.block_mask,
@@ -1624,18 +1619,18 @@ def _splash_attention_bwd_dq(
   num_scalar_prefetch = 3
 
   kernel_name = get_kernel_name(
-      dict(
-          block_q_dq=bq,
-          block_kv_dq=bkv,
-          q_layout=q_layout,
-          k_layout=k_layout,
-          v_layout=v_layout,
-      ),
       is_mqa=is_mqa,
       save_residuals=False,
       is_segmented=segment_ids is not None,
       phase="dq",
   )
+  metadata = {"xprof_metadata": json.dumps(dict(
+      block_q_dq=bq,
+      block_kv_dq=bkv,
+      q_layout=q_layout,
+      k_layout=k_layout,
+      v_layout=v_layout,
+  ))}
   with jax.named_scope(kernel_name):
     _, dq = pl.pallas_call(
         kernel,
@@ -1651,6 +1646,7 @@ def _splash_attention_bwd_dq(
         ),
         name=kernel_name,
         interpret=interpret,
+        metadata=metadata,
     )(
         mask_info.data_next,
         mask_info.block_mask,
@@ -2183,19 +2179,19 @@ def _splash_attention_bwd_dkv(
   num_scalar_prefetch = 3
 
   kernel_name = get_kernel_name(
-      dict(
-          block_q_dkv=bq,
-          block_kv_dkv=bkv,
-          block_kv_dkv_compute=bkv_compute,
-          q_layout=q_layout,
-          k_layout=k_layout,
-          v_layout=v_layout,
-      ),
       is_mqa=is_mqa,
       save_residuals=False,
       is_segmented=segment_ids is not None,
       phase="dkv",
   )
+  metadata = {"xprof_metadata": json.dumps(dict(
+      block_q_dkv=bq,
+      block_kv_dkv=bkv,
+      block_kv_dkv_compute=bkv_compute,
+      q_layout=q_layout,
+      k_layout=k_layout,
+      v_layout=v_layout,
+  ))}
   with jax.named_scope(kernel_name):
     _, _, _, dq_unreduced, dk, dv = pl.pallas_call(
         kernel,
@@ -2216,6 +2212,7 @@ def _splash_attention_bwd_dkv(
         ),
         name=kernel_name,
         interpret=interpret,
+        metadata=metadata,
     )(
         mask_info.data_next,
         mask_info.block_mask,
