@@ -27,19 +27,19 @@ import jax
 import jax.numpy as jnp
 from jax import lax
 from jax import tree_util
+from jax.experimental.sparse import _lowerings
 from jax.experimental.sparse._base import JAXSparse
 from jax.experimental.sparse import bcoo
 from jax.experimental.sparse.util import (
-    nfold_vmap, _count_stored_elements,
-    _csr_to_coo, CuSparseEfficiencyWarning, SparseInfo, Shape)
-from jax.util import split_list, safe_zip
+    nfold_vmap, _count_stored_elements, _csr_to_coo,
+    SparseEfficiencyWarning, CuSparseEfficiencyWarning, SparseInfo, Shape)
+from jax._src.util import split_list, safe_zip
 
 from jax._src import api_util
 from jax._src import config
 from jax._src import core
 from jax._src import dispatch
 from jax._src.lax.lax import DotDimensionNumbers, _dot_general_batch_dim_nums
-from jax._src.lib import gpu_sparse
 from jax._src.lib.mlir.dialects import hlo
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
@@ -144,7 +144,7 @@ def _bcsr_to_bcoo(indices: jax.Array, indptr: jax.Array, *,
 
 
 def _bcoo_to_bcsr(indices: Array, *, shape: Sequence[int],
-                  index_dtype: DTypeLike = jnp.int32) -> tuple[Array, Array]:
+                  index_dtype: DTypeLike) -> tuple[Array, Array]:
   """Given BCOO (indices), return BCSR (indices, indptr).
 
   Note: this assumes that ``indices`` are lexicographically sorted within each batch.
@@ -237,7 +237,9 @@ def _bcsr_fromdense_impl(mat, *, nse, n_batch, n_dense, index_dtype):
     raise ValueError("bcsr_fromdense: must have 2 sparse dimensions.")
   bcoo_mat = bcoo.bcoo_fromdense(mat, nse=nse, index_dtype=index_dtype,
                                  n_dense=n_dense, n_batch=n_batch)
-  indices, indptr = _bcoo_to_bcsr(bcoo_mat.indices, shape=mat.shape)
+  indices, indptr = _bcoo_to_bcsr(
+      bcoo_mat.indices, shape=mat.shape, index_dtype=index_dtype
+  )
   return bcoo_mat.data, indices, indptr
 
 
@@ -620,9 +622,9 @@ _bcsr_correct_out_of_bound_indices_lowered = mlir.lower_fun(
     _bcsr_correct_out_of_bound_indices, multiple_results=True)
 
 def _bcsr_dot_general_gpu_lowering(
-    csr_matvec_lowering, csr_matmat_lowering,
+    # csr_matvec_lowering, csr_matmat_lowering,
     ctx, lhs_data, lhs_indices, lhs_indptr, rhs, *, dimension_numbers,
-    preferred_element_type, lhs_spinfo: SparseInfo):
+    preferred_element_type, lhs_spinfo: SparseInfo, target_name_prefix):
 
   if not config.bcoo_cusparse_lowering.value:
     return _bcsr_dot_general_default_lowering(
@@ -674,22 +676,112 @@ def _bcsr_dot_general_gpu_lowering(
   lhs_data, lhs_indices = _bcsr_correct_out_of_bound_indices_lowered(
     ctx, lhs_data, lhs_indices, lhs_indptr, rhs, shape=lhs_spinfo.shape)
 
+  sub_ctx = ctx
   if rhs_aval.ndim == 1:
-    dot_general_fn = csr_matvec_lowering
-    x_dtype = 'x_dtype'
+    dot_general_fn = _lowerings._csr_spmv_gpu_lowering
   elif rhs_aval.ndim == 2:
-    dot_general_fn = csr_matmat_lowering
-    x_dtype = 'B_dtype'
+    dot_general_fn = _lowerings._csr_spmm_gpu_lowering
     if rhs_contract[0] == 1:
       rhs = hlo.transpose(rhs, permutation=mlir.dense_int_array([1, 0]))
+      *avals_in, rhs_aval = sub_ctx.avals_in
+      rhs_aval = core.ShapedArray(
+          shape=(rhs_aval.shape[1], rhs_aval.shape[0]), dtype=rhs_aval.dtype)
+      sub_ctx = sub_ctx.replace(avals_in=[*avals_in, rhs_aval])
   else:
     raise ValueError(f"rhs has to be 1d or 2d; get {rhs_aval.ndim}d.")
 
-  return [dot_general_fn(lhs_data, lhs_indices, lhs_indptr, rhs,
-                         shape=lhs_spinfo.shape, transpose=False,
-                         data_dtype=lhs_data_aval.dtype,
-                         index_dtype=lhs_indices_aval.dtype,
-                         **{x_dtype: rhs_aval.dtype})]
+  return dot_general_fn(sub_ctx, lhs_data, lhs_indices, lhs_indptr, rhs,
+                        shape=lhs_spinfo.shape, transpose=False,
+                        target_name_prefix=target_name_prefix)
+
+
+def _bcsr_dot_general_cpu_lowering(
+    # csr_matvec_lowering, csr_matmat_lowering,
+    ctx,
+    lhs_data,
+    lhs_indices,
+    lhs_indptr,
+    rhs,
+    *,
+    dimension_numbers,
+    preferred_element_type,
+    lhs_spinfo: SparseInfo,
+):
+
+  (lhs_contract, rhs_contract), (lhs_batch, rhs_batch) = dimension_numbers
+  lhs_data_aval, lhs_indices_aval, lhs_indptr_aval, rhs_aval = ctx.avals_in
+  props = _validate_bcsr(
+      lhs_data_aval, lhs_indices_aval, lhs_indptr_aval, lhs_spinfo.shape
+  )
+
+  use_default_lowering = False
+  dtype = lhs_data_aval.dtype
+  if lhs_batch or rhs_batch:
+    # TODO(willfroom): Add support for batched matrices.
+    use_default_lowering = True
+  elif lhs_data_aval.dtype != rhs_aval.dtype:
+    use_default_lowering = True
+  elif (
+      preferred_element_type is not None
+      and preferred_element_type != lhs_data_aval.dtype
+  ):
+    use_default_lowering = True
+  elif len(lhs_spinfo.shape) != 2 or rhs_aval.ndim not in [1, 2]:
+    # only matmat / matvec supported
+    use_default_lowering = True
+  elif props.n_batch or props.n_dense:
+    # batch and dense dimensions in BCSR not supported
+    use_default_lowering = True
+  elif list(lhs_contract) != [1] or list(rhs_contract) != [0]:
+    # TODO(willfroom): Add support for non-canonical dots.
+    use_default_lowering = True
+  elif lhs_indices_aval.dtype != lhs_indptr_aval.dtype:
+    warnings.warn(
+        "bcsr_dot_general cpu lowering not available, "
+        f" {lhs_indices_aval.dtype=} and {lhs_indptr_aval.dtype=} do not match."
+        " Falling back to default implementation.",
+        SparseEfficiencyWarning,
+    )
+    use_default_lowering = True
+  elif lhs_indices_aval.dtype not in [np.int32, np.int64]:
+    use_default_lowering = True
+    warnings.warn(
+        "bcsr_dot_general cpu lowering not available for"
+        f" {lhs_indices_aval.dtype=}. Falling back to default implementation.",
+        SparseEfficiencyWarning,
+    )
+  elif dtype not in [
+      np.int32,
+      np.int64,
+      np.float32,
+      np.float64,
+      np.complex64,
+      np.complex128,
+  ]:
+    # This would be supported if not for the dtype.
+    warnings.warn(
+        "bcsr_dot_general cpu lowering not available "
+        f"for {dtype=}. Falling back to default implementation.",
+        SparseEfficiencyWarning,
+    )
+    use_default_lowering = True
+
+  if use_default_lowering:
+    return _bcsr_dot_general_default_lowering(
+        ctx,
+        lhs_data,
+        lhs_indices,
+        lhs_indptr,
+        rhs,
+        dimension_numbers=dimension_numbers,
+        preferred_element_type=preferred_element_type,
+        lhs_spinfo=lhs_spinfo,
+    )
+
+  return _lowerings._csr_spmm_cpu_lowering(
+      ctx, lhs_data, lhs_indptr, lhs_indices, rhs
+  )
+
 
 _bcsr_dot_general_default_lowering = mlir.lower_fun(
     _bcsr_dot_general_impl, multiple_results=False)
@@ -697,20 +789,21 @@ mlir.register_lowering(
     bcsr_dot_general_p, _bcsr_dot_general_default_lowering)
 dispatch.simple_impl(bcsr_dot_general_p)
 
-if gpu_sparse.cuda_is_supported:
-  mlir.register_lowering(bcsr_dot_general_p,
-                          partial(_bcsr_dot_general_gpu_lowering,
-                                  gpu_sparse.cuda_csr_matvec,
-                                  gpu_sparse.cuda_csr_matmat),
-                          platform='cuda')
-if gpu_sparse.rocm_is_supported:
-  mlir.register_lowering(bcsr_dot_general_p,
-                          partial(_bcsr_dot_general_gpu_lowering,
-                                  gpu_sparse.rocm_csr_matvec,
-                                  gpu_sparse.rocm_csr_matmat),
-                          platform='rocm')
+mlir.register_lowering(bcsr_dot_general_p,
+                        partial(_bcsr_dot_general_gpu_lowering,
+                                target_name_prefix='cu'),
+                        platform='cuda')
+mlir.register_lowering(bcsr_dot_general_p,
+                        partial(_bcsr_dot_general_gpu_lowering,
+                                target_name_prefix='hip'),
+                        platform='rocm')
 
 
+if _lowerings.has_cpu_sparse:
+  mlir.register_lowering(
+      bcsr_dot_general_p, _bcsr_dot_general_cpu_lowering, platform="cpu"
+  )
+
 #----------------------------------------------------------------------
 # BCOO functions that maybe should be primitives?
 
@@ -867,7 +960,9 @@ class BCSR(JAXSparse):
       raise NotImplementedError(f"BSCR.from_bcoo requires n_sparse=2; got {arr.n_sparse=}")
     if not arr.indices_sorted:
       arr = arr.sort_indices()
-    indices, indptr = _bcoo_to_bcsr(arr.indices, shape=arr.shape)
+    indices, indptr = _bcoo_to_bcsr(
+        arr.indices, shape=arr.shape, index_dtype=arr.indices.dtype
+    )
     return cls((arr.data, indices, indptr), shape=arr.shape)
 
   @classmethod
