@@ -15,23 +15,21 @@
 
 from __future__ import annotations
 
-from collections.abc import Iterable, Iterator, Sequence
+from collections.abc import Callable, Iterable, Iterator, Sequence
 import functools
 import itertools
 import math
-from typing import Any, Callable, Union
-from typing import NamedTuple
+from typing import Any, NamedTuple
 
 import jax
 from jax import lax
 from jax import tree_util
 from jax._src import test_util as jtu
 from jax._src.lax.lax import DotDimensionNumbers
-from jax._src.lib import gpu_sparse
 from jax._src.typing import DTypeLike
 from jax.experimental import sparse
 import jax.numpy as jnp
-from jax.util import safe_zip, split_list
+from jax._src.util import safe_zip, split_list
 import numpy as np
 
 MATMUL_TOL = {
