@@ -34,4 +34,9 @@ from jax._src import (
     util as _util
 )
 register_backend_cache = _util.register_cache  # type: ignore
+
+from jax._src.lib import (
+    ifrt_proxy as ifrt_proxy
+)
+
 del _util
