@@ -15,6 +15,8 @@
 # Note: import <name> as <name> is required for names to be exported.
 # See PEP 484 & https://github.com/jax-ml/jax/issues/7570
 
+from collections.abc import Callable
+
 from jax._src.linear_util import (
   StoreException as StoreException,
   WrappedFun as WrappedFun,
@@ -24,5 +26,14 @@ from jax._src.linear_util import (
   transformation_with_aux as transformation_with_aux,
   transformation2 as transformation2,
   transformation_with_aux2 as transformation_with_aux2,
-  wrap_init as wrap_init,
+  # TODO(b/396086979): remove this once we pass debug_info everywhere.
+  wrap_init as _wrap_init,
+  _missing_debug_info as _missing_debug_info,
 )
+
+# Version of wrap_init that does not require a DebugInfo object.
+# This usage is deprecated, use api_util.debug_info() to construct a proper
+# DebugInfo object.
+def wrap_init(f: Callable, params=None, *, debug_info=None) -> WrappedFun:
+  debug_info = debug_info or _missing_debug_info("linear_util.wrap_init")
+  return _wrap_init(f, params, debug_info=debug_info)
