@@ -26,7 +26,13 @@ from jax._src.monitoring import (
     record_event_duration_secs as record_event_duration_secs,
     record_event_time_span as record_event_time_span,
     record_event as record_event,
+    record_scalar as record_scalar,
     register_event_duration_secs_listener as register_event_duration_secs_listener,
     register_event_listener as register_event_listener,
     register_event_time_span_listener as register_event_time_span_listener,
+    register_scalar_listener as register_scalar_listener,
+    unregister_event_duration_listener as unregister_event_duration_listener,
+    unregister_event_listener as unregister_event_listener,
+    unregister_event_time_span_listener as unregister_event_time_span_listener,
+    unregister_scalar_listener as unregister_scalar_listener,
 )
