@@ -31,6 +31,7 @@ from jax._src.nn.functions import (
   leaky_relu as leaky_relu,
   log_sigmoid as log_sigmoid,
   log_softmax as log_softmax,
+  logmeanexp as logmeanexp,
   logsumexp as logsumexp,
   standardize as standardize,
   one_hot as one_hot,
@@ -52,4 +53,5 @@ from jax._src.nn.functions import (
   swish as swish,
   squareplus as squareplus,
   mish as mish,
+  log1mexp as log1mexp,
 )
