@@ -19,6 +19,7 @@ The :mod:`jax.tree` namespace contains aliases of utilities from :mod:`jax.tree_
 
 from jax._src.tree import (
     all as all,
+    broadcast as broadcast,
     flatten_with_path as flatten_with_path,
     flatten as flatten,
     leaves_with_path as leaves_with_path,
@@ -26,6 +27,7 @@ from jax._src.tree import (
     map_with_path as map_with_path,
     map as map,
     reduce as reduce,
+    reduce_associative as reduce_associative,
     structure as structure,
     transpose as transpose,
     unflatten as unflatten,
